#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_27/sys/src/install/perl/SAPDB/Install/Control/StaticText.pm#1 $
# $DateTime: 2006/04/27 14:44:55 $
# $Change: 124736 $
#
# Desc: 

package SAPDB::Install::Control::StaticText;

use Wx::Event qw (EVT_PAINT EVT_SIZE);
use Wx qw(wxDECORATIVE wxNORMAL wxBOLD
		wxSOLID wxBLUE wxWHITE wxSYS_SYSTEM_FONT
		wxSYS_DEFAULT_GUI_FONT);

use base SAPDB::Install::Helpers::PassiveWin;

sub new{
	my($this ) = shift->SUPER::new(@_);
	$this->SetBackgroundColour($this->GetParent()->GetBackgroundColour());
	#$this->{'font'} = Wx::Font->new(15,$^O =~ /mswin/i ? wxNORMAL : wxDECORATIVE, wxNORMAL, wxBOLD, 0 );
	if($^O =~ /mswin/i){
		$this->{'font'} = Wx::SystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
	}
	else{
		$this->{'font'} = Wx::SystemSettings::GetFont(wxSYS_SYSTEM_FONT);
	}
	$this->{'brush'} = Wx::Brush->new($this->GetParent()->GetBackgroundColour(),wxSOLID);
	$this->{'invers'} = 0;
	$this->{'auto_resize'} = 1;
	$this->{'auto_vresize'} = 1;
	$this->{'mdc'} = Wx::MemoryDC->new();
	$this->{'mdc'}->SelectObject(Wx::Bitmap->new(1,1));
	$this->{'mdc'}->SetFont($this->{'font'});
	EVT_PAINT($this,\&OnPaint);
	EVT_SIZE($this,\&OnSize);
	return $this;
}

sub OnSize{
	my	($self,$evt) = @_;
	if($self->{'auto_split'}){
		$self->setText($self->splitText($self->{'text'}));
	}
}


sub splitText{
	my ($self,$text) = @_;
	my ($x,$y) = $self->GetSizeWH;
	my $new_line = '';
	my @result; 
	foreach my $word (split(' ',$text )){
		my $tmp_line = $new_line;
		$new_line .= "$word ";
		my ($nx) = $self->{'mdc'}->GetTextExtent($new_line);
		if(($nx) > $x){
			push @result,$tmp_line;
			$new_line = "$word ";
		}
	}
	push @result,$new_line;
	return join("\n",@result);	
}

sub cutText{
	my ($self,$text) = @_;
	my ($x,$y) = $self->GetSizeWH;
	my $replace = ' ... ';
	my $new_line = '';
	my $len = length($text);
	my $len_replace = length($replace);
	my ($nx) = $self->{'mdc'}->GetTextExtent($text);
	if($nx > $x){
		while(1){
				my $new_text = $text;
				my $pos = int(($len - $len_replace) / 2);
				$len_replace++;
				$new_text =~ s/^(.{$pos}).{$len_replace}/$1$replace/;
				($nx) = $self->{'mdc'}->GetTextExtent($new_text);
				if($nx <= $x){
					$text = $new_text;
					last;
				}
		}
	}
	return $text;
	
}




sub setText{
	my ($self,$text,$force_repaint) = @_;
	$self->{'text_set'} = 1;
	if($self->{'auto_split'}){
		$text = $self->splitText($text);
	}
	if($self->{'auto_cut'}){
		$self->{'auto_resize'} = 0;
		$text = $self->cutText($text);
	}
	if ($text ne $self->{'text'} or $force_repaint){
		my @size = $self->getTextExtend($text);
		if($self->{'auto_resize'}){
			$self->SetSize([$size[0],$size[1]]);
		}
		elsif($self->{'auto_vresize'}){
			$self->SetSize([$self->GetSize->GetWidth,$size[1]]);	
		}
		$self->{'current_textextent'} = \@size;
		
		$self->{'text'} = $text;
		$self->Render;
		
		$self->Refresh;
	}
}

sub getTextExtend{
	my ($self,$text) = @_;
	my $line = 0;
	my $xmax = 0;
	$self->{'mdc'}->SetFont($self->{'font'});
	foreach my $text_part (split("\n",$text)){
		my ($x,$y) = $self->{'mdc'}->GetTextExtent($text_part);
		$xmax = $x if ($x > $xmax);
		$line++;
	}
	return (0,0) if $xmax == 0;
	my ($dummy,$ysize) = $self->{'mdc'}->GetTextExtent('Ag');
	return ($xmax, $line * $ysize);
}


sub Render{
	my ($self) = @_;
	$self->{'mdc'}->SelectObject(Wx::Bitmap->new($self->GetSizeWH));
	$self->{'mdc'}->BeginDrawing() if $^O =~ /mswin/i;
	$self->{'mdc'}->SetBackground($self->{'brush'});
	$self->{'mdc'}->SetTextForeground($self->{'text_colour'}) if defined $self->{'text_colour'};
  	$self->{'mdc'}->Clear;
	$self->{'mdc'}->SetFont($self->{'font'});
	my $line = 0;
	my ($dummy,$ysize) = $self->{'mdc'}->GetTextExtent('Ag');
	foreach my $text_part (split("\n",$self->{'text'})){
		$self->{'mdc'}->DrawText($text_part,0,$line * $ysize );
		$line++;
	}
	$self->{'mdc'}->EndDrawing() if $^O =~ /mswin/i;
	if(defined $self->{'imdc'}){
		$self->{'imdc'}->SelectObject(Wx::Bitmap->new($self->GetSizeWH));
		$self->{'imdc'}->BeginDrawing() if $^O =~ /mswin/i;
		
		if(defined $self->{'ibg_colour'}){
			$self->{'imdc'}->SetBackground(Wx::Brush->new($self->{'ibg_colour'},wxSOLID));
		}
		else{
			$self->{'imdc'}->SetBackground(Wx::Brush->new(wxBLUE,wxSOLID));
		}
		if(defined $self->{'itext_colour'}){
			$self->{'imdc'}->SetTextForeground($self->{'itext_colour'});
  		}
		else{
			$self->{'imdc'}->SetTextForeground(wxWHITE);
		}
		$self->{'imdc'}->Clear;
		$self->{'imdc'}->SetFont($self->{'font'});
		my $line = 0;
		my ($idummy,$iysize) = $self->{'imdc'}->GetTextExtent('Ag');
		foreach my $text_part (split("\n",$self->{'text'})){	
			$self->{'imdc'}->DrawText($text_part,0,$line * $iysize);
			$line++;
		}
		$self->{'imdc'}->EndDrawing() if $^O =~ /mswin/i;
	}	
}

sub SetLabel{
	my ($self,$label) = @_;
	$self->setText($label);
	return $self->SUPER::SetLabel($label);
}


sub setInvers{
	my ($self) = @_;
	unless(defined $self->{'imdc'}){
		$self->{'imdc'} = Wx::MemoryDC->new();
		$self->Render;
	}
	$self->{'invers'} = ($self->{'invers'} xor 1);
	$self->Refresh;
}

sub OnPaint{
	my ($self,$evt) = @_;
	return unless defined $self->{'mdc'};
	unless(defined $self->{'text'}){
		$evt->Skip(1);
		return undef;
	}
	$self->Render unless $self->{'text_set'};
	my $pdc = Wx::PaintDC->new($self);
	if($self->{'invers'}){
		$pdc->Blit(0,0,$self->GetSizeWH,$self->{'imdc'},0,0);
	}
	else{
		$pdc->Blit(0,0,$self->GetSizeWH,$self->{'mdc'},0,0);
	}
	$evt->Skip(1);
}

sub Destroy{
	my ($self) = @_;
	delete $self->{'mdc'};
	delete $self->{'mdci'} if exists $self->{'mdci'};
	delete $self->{'font'};
	delete $self->{'brush'};
	delete $self->{'text'};
	$self->SUPER::Destroy;
}


sub DESTROY{
	my ($self) =  @_;
	print ">>>> DESTROY $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}

1;