#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_27/sys/src/install/perl/SAPDB/Install/Service/Communication.pm#1 $
# $DateTime: 2006/04/27 14:44:55 $
# $Change: 124736 $
#
# Desc: 

package SAPDB::Install::Service::Communication;

use SAPDB::Install::Base;
use SAPDB::Install::Registry;
use SAPDB::Install::System;
use SAPDB::Install::Tools;

our @ISA = qw(SAPDB::Install::Base);

our %member = (
	'port' => undef,
	'image' => undef,
	'state' => undef,
	'last_error' => undef,
	'exec_rc' => undef,
	'command_out' => undef	
);

our ($indep_data_path,$indep_prog_path) = readIndepPath(); 

sub new{
	my %lmember = %member;
	unless($indep_prog_path =~ /\S/){
		($indep_data_path,$indep_prog_path) = readIndepPath(); 
	}
	my $self = bless(\%lmember,shift);
	if($^O =~ /mswin/i){
		my $path = getRealPathName($indep_prog_path);
		$self->image('"'.$path.'\bin\x_server.exe"');
	}
	else{
		$self->image($indep_prog_path.'/bin/x_server');
	}	
	$self->refresh;
	return $self;
}


sub exec{
	my ($self,$command) = @_;
	my $file = $self->image;
	$file =~ s/^\"//;
	$file =~ s/\"$//;
	unless (-f $file){
 		$self->last_error('x_server executable ('.$self->image.') not found');
 		return undef;
 	}	
	my $get_output = sub {
		my ($text) = @_;
		$self->command_out($text);	
		return 1;
 	};
 	my $rc = callsystem($self->image.' '.$command,$get_output,1);
	$self->exec_rc($rc);
	if(not defined $rc or $rc != 0){
		if(defined $rc){
			$self->last_error("x_server $command returned $rc: ".$self->command_out);
		}
		else{
			$self->last_error("x_server $command failed");
		}
		return undef;
	}
	$self->refresh;
	return 1;
}


sub start{
	my ($self) = @_;
	unless($self->state->{'is_running'}){	
		unless($self->exec('start')){
			$self->last_error('cannot start xserver: '.$self->last_error);
			return undef;
		}
		unless($self->state->{'is_running'}){
			$self->last_error('cannot start xserver: x_ping failed');
			return undef;
		}
	}
	return 1;
}

sub restart{
	my ($self) = @_;
	unless($self->stop){
		return undef;
	}
	unless($self->start){
		return undef;
	}
	return 1;
}

sub refresh{
	my ($self) = @_;
	$self->state(xserverInfo());
	return 1;
}


sub stop{
	my ($self) = @_;
	if($self->state->{'is_running'}){
		unless($self->exec('stop')){
			$self->last_error('cannot stop x_server: '.$self->last_error);
			return undef;
		}
	}
	return 1;

}

sub update{
	my ($self) = @_;
	if($^O =~ /mswin/i){
		$self->last_error('communication server update not yet implemented for this platform');
		return undef;
	}
	if($self->{'is_updatable'}){
		unless($self->exec('update')){
			$self->last_error('cannot update x_server: '.$self->last_error);
			return undef;
		}
	}
	return 1;
}


1;