package SAPDB::Install::Templates::DropDBWizard;

use strict;
use base qw(Wx::Wizard );
use Wx qw(:sizer wxDefaultPosition wxDefaultSize
          wxDEFAULT_DIALOG_STYLE wxRESIZE_BORDER wxID_OK);

sub new {
  my ($class,$frame,$app) = @_;
  my $this = $class->SUPER::new($frame,-1,"Drop MaxDB Instance Wizard");
  $this->{App} = $app;
  return $this;
}



sub HasNextPage{
  my ($this,$current) = @_;
  return $current->HasNext();
}
sub HasPrevPage{
  my ($this,$current) = @_;
  return $current->HasPrev();
}



sub GetFirst(){
  my ($this) = @_;
  my $next = SAPDB::Install::Templates::DropDBPage1->new($this); 
  return $next;
}


package SAPDB::Install::Templates::DropDBPage;
use strict;
use base qw(Wx::WizardPage);

sub new {
  my ($class,$wizard) = @_;
  my $this = $class->SUPER::new( $wizard);
  $this->{CurrentWizard}=$wizard;
  $this->{Instances}=[];
  return $this;
}

sub getCurrentWizard{
  return shift->{CurrentWizard};
}



package SAPDB::Install::Templates::DropDBPage1;

use strict;
use base qw( SAPDB::Install::Templates::DropDBPage );
use Wx qw(:sizer wxDefaultPosition wxDefaultSize  wxICON_ERROR wxOK);
use Wx::Event qw(EVT_CHECKLISTBOX 
                 EVT_WIZARD_PAGE_CHANGED EVT_WIZARD_PAGE_CHANGING EVT_WIZARD_CANCEL );
use SAPDB::Install::Registry;


sub new {
  my ($class,$wizard) = @_;
  my $this = $class->SUPER::new( $wizard);
  my $box1 = Wx::BoxSizer->new( wxVERTICAL );

  
  my %aHash = getInstances();
  my @aList =  keys %aHash;
  
  if( @aList){
    $this->{hasNextKey} = 1;
    my $checkListBox = Wx::CheckListBox->new
      ( $this, -1, wxDefaultPosition, wxDefaultSize,
        \@aList
        #            [ qw (
        #                  MAXDB1 MIDB1 HOTELDB
        #                 ) ]
      );
    $this->{CheckListBox} = $checkListBox;
    EVT_CHECKLISTBOX( $this, $checkListBox, \&OnCheck );
    EVT_WIZARD_PAGE_CHANGING( $this, $wizard, \&OnWizardPageChanging );
    
    my $staticText = Wx::StaticText->new
      ( $this, -1, 'Choose the database instances to delete: ',
        wxDefaultPosition,wxDefaultSize
      );
    $box1->Add($staticText,0,wxEXPAND|wxBOTTOM,10);
    $box1->Add($checkListBox,0,wxEXPAND);
  }
  else{
    $this->{hasNextKey} = 0;
    my $staticText = Wx::StaticText->new
      ( $this, -1, 'No database instances found!',
        wxDefaultPosition,wxDefaultSize
      );
    $box1->Add($staticText,0,wxEXPAND|wxBOTTOM,10);
  }
  $this->SetAutoLayout( 1 );
  $this->SetSizer( $box1 );
  $box1->Fit( $this );
  $box1->SetSizeHints( $this );
  return $this;
}


sub HasNext{
  my $instanceArray = shift->{hasNextKey};
}
sub HasPrev{
  return 0;
}


sub getInstanceArray{
  return shift->{Instances};
}

sub setInstancesArray{
  my ($this,$instanceList) = @_;
  $this->{Instances}= $instanceList;
}

sub OnWizardPageChanging{
  my ($this,$event) = @_;
  my $instances = $this->getInstanceArray();
  unless(defined @$instances){
    my $ result = Wx::MessageBox
      ( "Choose at least one database!" , "Nothing selected!", wxICON_ERROR | wxOK, $this );
    $event->Veto();
  }
}


sub DeleteFromCheckList{
  my ($this,$instance) = @_;
  my $dbName = $instance->{Name};
  my $dbList = $this->{CheckListBox};
  $dbList->Delete( $dbList->FindString($dbName));
  return $this->UpdateSelectedList();
}

sub UpdateSelectedList{
  my $this = shift;
  my $checkListBox = $this->{CheckListBox};
  my @newList;
  for(my $i=0; $i < $checkListBox->GetCount();++$i){
    if ( $checkListBox->IsChecked( $i)) {
      my $db = $checkListBox->GetString($i);
      push @newList,( {Name => $db});
    }
  }
  my $newRef = \@newList;
  $this->setInstancesArray( $newRef);
  return $newRef;
}

  sub OnCheck {
    my( $this, $event ) = @_;
    $this->UpdateSelectedList();
  }
  sub GetNext(){
    my ($this) = @_;
    my $next = undef; 
    if($this->HasNext()){
      my $wizard = $this->getCurrentWizard();
      $next = SAPDB::Install::Templates::DropDBPage2->new($wizard,$this); 
    }
    return $next;
  }
  sub GetPrev(){
    my ($this) = @_;
    return undef;
  }

  package SAPDB::Install::Templates::DropDBPage2;

  use strict;
  use base qw(SAPDB::Install::Templates::DropDBPage);
  use Wx qw(:sizer wxDefaultPosition wxDefaultSize
            wxDEFAULT_DIALOG_STYLE wxRESIZE_BORDER wxID_OK wxALL wxTE_PASSWORD
            wxICON_QUESTION  wxYES_NO  wxYES  wxTOP wxLEFT wxBOTTOM wxRIGHT
            wxALIGN_CENTRE wxALIGN_CENTRE_VERTICAL wxICON_ERROR wxOK wxHOURGLASS_CURSOR 
            wxCURSOR_ARROW wxCURSOR_PENCIL wxSTANDARD_CURSOR

           );
  use Wx::Event qw( EVT_WIZARD_PAGE_CHANGED EVT_WIZARD_PAGE_CHANGING EVT_WIZARD_CANCEL );

  use SAPDB::Install::Templates::Simple;


  sub new {
    my ($class,$wizard,$instanceSelection) = @_;
    my $this = $class->SUPER::new( $wizard);
    $this->{InstanceSelection} = $instanceSelection;
    my $instances = $instanceSelection->getInstanceArray();
    my $instance =$instances->[0];
    my $dbname = $instance->{Name};
    unless(defined $dbname){
      return $this;
    }
    my $s1 = Wx::StaticText->new( $this, -1, "Enter authorization data for  $dbname:" );
    my $dbmName = Wx::StaticText->new( $this, -1, "Database manager:",wxDefaultPosition,wxDefaultSize,
                                     wxALIGN_CENTRE);
    my $dbmNameEntry = Wx::TextCtrl->new( $this, -1, "" );
    $this->{dbm}  = $dbmNameEntry;
    my $dbmPass = Wx::StaticText->new( $this, -1, "Password:" );
    my $dbmPassEntry = Wx::TextCtrl->new( $this, -1, "",wxDefaultPosition,wxDefaultSize,wxTE_PASSWORD  );
    $this->{dbmPass} = $dbmPassEntry;

    my $box1 = Wx::BoxSizer->new( wxVERTICAL );
    $box1->Add($s1,0,wxBOTTOM,15);


    my $gridSizer = Wx::FlexGridSizer->new( 2, 2, 0, 5 );
    $gridSizer->AddGrowableCol(1);
    $gridSizer->Add( $dbmName, 0, wxBOTTOM | wxALIGN_CENTRE_VERTICAL, 3 );
    $gridSizer->Add( $dbmNameEntry, 0, wxBOTTOM| wxALIGN_CENTRE_VERTICAL|wxGROW, 3 );
    $gridSizer->Add( $dbmPass, 0, wxBOTTOM | wxALIGN_CENTRE_VERTICAL, 3 );
    $gridSizer->Add( $dbmPassEntry, 0, wxBOTTOM | wxALIGN_CENTRE_VERTICAL|wxGROW, 3 );
    $box1->Add($gridSizer,0,wxGROW);
    $this->SetAutoLayout( 1 );
    $this->SetSizer( $box1 );
    $box1->Fit( $this );
    $box1->SetSizeHints( $this );
    return $this;
  }
  sub HasNext{
      return 1;
  }
  sub HasPrev{
    return shift->{InstanceSelection};
  }
  sub GetNext(){
    my ($this) = @_;
    my $instanceSelection = $this->{InstanceSelection};
    my $instances = $instanceSelection->getInstanceArray();
    my $instance = $instances->[0];
    my $dbName = $instance->{Name};
    my $dbm = $this->{dbm}->GetValue();
    my $dbmPass = $this->{dbmPass}->GetValue();
    if( "" eq $dbm){
       Wx::MessageBox
        ( "Please enter name of database administrator" ,
          "Missing entry", wxICON_ERROR | wxOK, $this );
       return $this;
     }

    if( "" eq $dbmPass){
       Wx::MessageBox
        ( "Please enter password of database administrator" ,
          "Missing entry", wxICON_ERROR | wxOK, $this );
       return $this;
     }

    my $wizard = $this->getCurrentWizard();
    my $db= SAPDB::Install::Templates::Simple::new();
    $db->setDBName($dbName);
    $db->setDBM($dbm);
    $db->setDBMPass($dbmPass);
    $db->setAsyncCallback(
                          sub{
                            my ($time,$message) = @_;
#                            print "$time: $message\n";
                            $wizard->Update();
                            my $app = $wizard->{App};
                            if(defined $app){
                              while($app->Pending()){
				$app->Dispatch();
                              }
                            }
                          });

    #Do the deletion:
    $wizard->SetCursor(  wxHOURGLASS_CURSOR );
#    foreach( 1 .. 10 ){
#      sleep 1;
#    }
#    my $result = undef;
    my $result = $db->deleteInstance();
    $wizard->SetCursor( wxSTANDARD_CURSOR);

    my $next = undef;


    if( defined $result){
      $instances = $instanceSelection->DeleteFromCheckList($instance);
      if( @$instances){
        $next =
          SAPDB::Install::Templates::DropDBPage2->new
              ($this->getCurrentWizard(),$instanceSelection); 
      }
      else{
        $next =
          SAPDB::Install::Templates::DropDBPage3->new ($this->getCurrentWizard()); 
      }
    }else{

      my $msg = $db->getLastDBMError();
      unless(defined $msg){
        $msg = $db->getLastDBMMessage();
      }
      unless(defined $msg){
        $msg = "Unknown error deleting database instance!";
      }
       Wx::MessageBox
        ( $msg , "Error deleting database", wxICON_ERROR | wxOK, $this );



      $next = $this;
    }
    return $next;
  }


  sub GetPrev(){
    my ($this) = @_;
    my $instanceSelection = $this->{InstanceSelection};
    return $instanceSelection;
  }

  package SAPDB::Install::Templates::DropDBPage3;

  use strict;
  use base qw(SAPDB::Install::Templates::DropDBPage);
  use Wx qw(:sizer wxDefaultPosition wxDefaultSize
            wxDEFAULT_DIALOG_STYLE wxRESIZE_BORDER wxID_OK wxALL wxTE_PASSWORD
            wxICON_QUESTION  wxYES_NO  wxYES  wxTOP wxLEFT wxBOTTOM wxRIGHT
            wxALIGN_CENTRE wxALIGN_CENTRE_VERTICAL
           );



  sub new {
    my ($class,$wizard) = @_;
    my $this = $class->SUPER::new( $wizard);
    my $s1 = Wx::StaticText->new( $this, -1, "All instances deleted" );

    my $box1 = Wx::BoxSizer->new( wxVERTICAL );
    $box1->Add($s1,0,wxBOTTOM,15);
    $this->SetAutoLayout( 1 );
    $this->SetSizer( $box1 );
    $box1->Fit( $this );
    $box1->SetSizeHints( $this );
    return $this;
  }
  sub HasNext{
      return 0;
  }
  sub HasPrev{
    return 0;
  }
  sub GetNext(){
    return undef;
  }
  sub GetPrev(){
    return undef;
  }

  1;

  # Local Variables: ***
  # perl-dbg-flags:"-w      -I../../../../../../wrk/script -I../../../sdbrun  " ***
  # sdbinst-defines: "-define SCRIPTING=YES -define PERL5.6.1=YES" ***
  # sdbinst-testfile: "WxTest.pl" ***
  # sdbinst-script-directory:  "sys/wrk/script/SAPDB/Install/Templates" ***
  # End: ***
