use strict;

package SAPDB::Install::Templates::Properties;
require Exporter;

our @ISA = qw( Exporter );
#our @EXPORT = qw( MergeKeys );

my  $INVALID = "SAPDB::Install::Properties::Invalid";
sub AUTOLOAD {
  our $AUTOLOAD;
  my $retVal =  Setter($AUTOLOAD,@_);
  if (not defined $retVal or $INVALID ne $retVal) {
    return $retVal;
  }
  $retVal =  Getter($AUTOLOAD,@_);
  if (not defined $retVal or $INVALID ne $retVal) {
    return $retVal;
  }
  $retVal =  Validator($AUTOLOAD,@_);
  if (not defined $retVal or $INVALID ne $retVal) {
    return $retVal;
  }
  unless($AUTOLOAD =~ /DESTROY$/){
    my ($package,$file,$line,$subroutine) = caller;
    $package = $package ? $package : "<UNKNOWN PACKAGE>";
    $file = $file ? $file : "<UNKNOWN FILE>";
    $line = $line ? $line : "<UNKNOWN LINE>";
    $subroutine = $subroutine ? $subroutine : "<UNKNOWN LINE>";
    die "No function $AUTOLOAD at $subroutine in '$file', line $line\n";
  }
}


#set initial values, if there are any
sub initData{
  my $this = shift;
  my $keys = $this->{'KEYS'};
  foreach my $aKey (keys %$keys){
    my $initValue = $this->GetInitial($aKey);
    unless(defined $initValue and $INVALID eq $initValue){
      $this->Set($aKey,$initValue);
    }
  }
}



my $listeners = 'SAPDB::Install::Templates::Properties::Listeners';
sub GetListeners{
  my ($this,$key)  = @_;
  $this->{$listeners} or $this->{$listeners}={};
  my $list = $this->{$listeners};
  $list->{$key} or $list->{$key}=[];
  return $list->{$key};
}


sub Setter{
  my ($setter,$this,$newVal) = @_;
  unless($setter =~/^.*::set(\w+)$/){
    return $INVALID;
  }
  my $aName = $1;
  my $key = $this->GetKey($aName);
  unless($key){
    dieWithStack("ERROR: Cannot set value for unknown key '$aName':");
  }
  return $this->Set($key,$newVal);
}

sub dieWithStack{
  my ($msg) = @_;
  $msg = "\n". $msg . "\n";
  my @c;
  my $i=0;
  while(@c = caller($i++)){
    $msg .= formatStack(@c) . "\n";
  }
  die $msg;
}

sub formatStack{
  my ($package,$file,$line,$subroutine) = @_;
  if($package){
    $package =~ /^.*::([^:]+)$/;
    if($1){
      $package = $1;
    }
  }
  else{
    $package= "<UNKNOWN>";
  }
  if($file){
  }
  else{
    $file= "<UNKNOWN>";
  }
  if($line){
  }
  else{
    $line= "<UNKNOWN>";
  }
  if($subroutine){
    $subroutine =~ /^.*::([^:]+)$/;
    if($1){
      $subroutine = $1;
    }
  }
  else{
    $subroutine= "<UNKNOWN>";
  }
  return $subroutine . " in " . $file . " line " . $line;
}



sub Set{
  my ($this,$key,$newVal) = @_;
  $this->{$key}=$newVal;
  return $newVal;
}



sub Getter{
  my ($getter,$this) = @_;
  my $retVal=undef;
  unless($getter =~/^.*::get(\w+)$/){
    return $INVALID;
  }
  my $aName = $1;
  my $key = $this->GetKey($aName);
  unless($key){
    dieWithStack("ERROR: Cannot get value for unknown key '$aName':");
  }
  return $this->Get($key);
}

sub Get{
  my ($this,$key) = @_;
  my $retVal ;
  if(exists  $this->{$key}){
    $retVal = $this->resolveValue($key,$this->{$key});
  }
  else{
    $retVal = undef;
    my $aDefault = $this->GetDefault($key);
    unless($INVALID eq $aDefault){
      $retVal = $this->resolveValue($key,$aDefault);
    }
  }
  return $retVal;
}








sub resolveValue{
  my ($this,$key,$retVal,$noSubst) = @_;
  my $isRef = ref $retVal;
  if ($isRef) {
    if ( 'CODE' eq $isRef) {
      $retVal = &$retVal($this,$key);
    }
  } elsif (defined $retVal and not $noSubst) {
    $retVal = $this->recursiveSubstitute($retVal);
    my $type = $this->GetType($key);
    $type and $type eq 'PATH' and $retVal = slashFix($retVal);
  }
  return $retVal;
}



sub recursiveSubstitute{
  my ($this,$val) = @_;
  my $retVal = $val;
  while ( $retVal =~ /\${(\w+)}/) {
    my $aVar = $1;
    my $aKey = $this->GetKeyIgnoreCase($aVar);
    unless($aKey){
      dieWithStack("ERROR: Cannot substitute value for unknown key '$aKey':");
    }
    my $aVal = $this->Get($aKey);
    $aVal = defined $aVal ? $aVal : "";
    $retVal =~ s/\${$aVar}/$aVal/g;
  }
  return $retVal;
}




sub parseLine{
  my ($this,$aLine) = @_;
  if( $aLine =~ /^\s*(\w+)\s+(.*\S|.*)\s*$/){
    my $aKey = $this->GetKeyIgnoreCase($1);
    if($aKey){
      my $aValue = $2;
      my $parser = $this->GetParser($aKey);
      if($parser){
        $aValue = &$parser($this,$aKey,$aValue);
      }
      if($this->GetKey($aKey)){
        $this->Set($aKey,$aValue);
      }
    }
  }
  else{
  }
}

sub parseFile{
  my ($this,$aFile) = @_;
  my $retVal = 0;
  $aFile or return "No input file specified!\n";
  open INFILE , "<$aFile" or return $!;
  $this->parseStream(*INFILE);
  close INFILE;
  return $retVal
}

sub parseStream{
  my ($this,$stream) = @_;
  my $aLine = '';
  while (<$stream>) {
    /^\s*#/ and next;
    chomp;
    s/\r$// unless $^O =~ /mswin/i;
    $aLine .= $_;
    if (/\\$/) {
      $aLine =~ s/\\$//;
    } else { 
      $this->parseLine($aLine);
      $aLine='';
    }
  }
  $aLine and $this->parseLine($aLine);
}

sub GetLineForKey{
  my ($this,$theKey) = @_;
  my $definition = $this->{'KEYS'}->{$theKey};
  my $retVal = undef;
  if( $definition ){
    my $value;
    my $formatter = $definition->{'FORMATTER'};
    if(exists  $this->{$theKey}){
      $value = $this->resolveValue($theKey,$this->{$theKey},"noSubstitute");
    }
    if( $formatter){
      $retVal = &$formatter($this,$theKey,$value);
    }
    else{
      $value = defined $value?$value : "";
      $retVal = (uc $theKey) . " " . $value . "\n";
    }
  }
  return $retVal;
}




sub GetLines{
  my ($this) = @_;
  my  @retVal;
  my $keys = $this->{'KEYS'};
  foreach my $aKey (keys %$keys){
    push @retVal, ($this->GetLineForKey($aKey));
  }
  return @retVal;
}
sub saveToFile{
  my ($this,$file) = @_;
  my $retVal=0;
  my @lines = $this->GetLines();
  open OUTFILE , ">$file" or return $!;
  print OUTFILE @lines;
  close OUTFILE;
  return $retVal;
}


#Validator liefert 0 bei Erfolg, ungleich null bei Fehler
sub Validator{
  my ($validator,$this,$newVal) = @_;
  unless($validator =~/^.*::validate(\w+)$/){
    return $INVALID;
  }
  my $key = $this->GetKey($1);
  return $this->ValidateKey($newVal,$key);
}


#Validator liefert 0 bei Erfolg, ungleich null bei Fehler
sub ValidateKey{
  my ($this,$newVal,$key) = @_;
  my $retVal=0;
  my $aVal = $this->GetValidator($key);
  if ($aVal) {
    my $type =  ref $aVal;
    if ('CODE' eq $type) {
      $retVal = &$aVal($this,$newVal,$key);
    } else {
      if (not defined $newVal) {
        $retVal =  "Invalid value '<UNDEFINED>' for property '$key'";
      } elsif ($newVal =~ /$aVal/) {
        $retVal=0;
      } else {
        $retVal =  "Invalid value '$newVal' for property '$key'";
      }
    }
  }
  return $retVal;
}                               #validateKey

sub GetKey{
  my ($this,$val) = @_;
  my $key = lcfirst $val;
  my $retVal = undef;
  if (exists $this->{'KEYS'}->{$key}) {
    $retVal = $key;
  }
  return $retVal;
}

sub GetKeyIgnoreCase{
  my ($this,$val) = @_;
  my $key = lc $val;
  my $retVal = undef;
  foreach(keys %{$this->{'KEYS'}}){
    if(lc $_ eq $key){
      $retVal = $_;
      last;
    }
  }
  return $retVal;
}





sub GetValidator{
  my ($this,$key) = @_;
  return $this->GetMetaProperty($key,'VALIDATOR');
}
sub GetDefault{
  my ($this,$key) = @_;
  my $metadata = $this->{'KEYS'}->{$key};
  my $retVal ; 
  if(exists $metadata->{'DEFAULT'}){
    $retVal = $metadata->{'DEFAULT'};
  }
  else{
    $retVal =  $INVALID;
  }
  return $retVal;
}
sub GetFormatter{
  my ($this,$key) = @_;
  return $this->GetMetaProperty($key,'FORMATTER');
}

sub GetInitial{
  my ($this,$key) = @_;
  unless($this->ExistsInitial($key)){
    return $INVALID;
  }
  my $retVal = $this->GetMetaProperty($key,'INITIAL');
  return $this->resolveValue($key,$retVal);
}
sub GetType{
  my ($this,$key) = @_;
  return $this->GetMetaProperty($key,'TYPE');
}
sub GetParser{
  my ($this,$key) = @_;
  return $this->GetMetaProperty($key,'PARSER');
}

sub GetMetaProperty{
  my ($this,$key,$name) = @_;
  unless(exists $this->{'KEYS'}->{$key}){
    return undef;
  }
  unless(exists $this->{'KEYS'}->{$key}->{$name}){
    return undef;
  }
  return $this->{'KEYS'}->{$key}->{$name};
}

sub ExistsInitial{
  my ($this,$key) = @_;
  return $this->ExistsMetaProperty($key,'INITIAL');
}
sub ExistsMetaProperty{
  my ($this,$key,$name) = @_;
  unless(exists $this->{'KEYS'}->{$key}){
    return 0;
  }
  unless(exists $this->{'KEYS'}->{$key}->{$name}){
    return 0;
  }
  return 1;
}

sub MergeKeys{
  my ($this,$newKeys) = @_;
  foreach my $aKey (keys %$newKeys){
    my $aValue = $newKeys->{$aKey};
    $this->{'KEYS'}->{$aKey}=$aValue;
  }
}


sub myKeys{
  return {};
}



sub Validate{
  my $this =  shift;
  my %retVal;
  foreach my $key (keys %$this) {
    unless( $key eq 'KEYS'){
      my $val = $this->Get($key);
      my $res = $this->ValidateKey($val,$key);
      if ($res) {
        $retVal{$key}=$res;
      }
    }
  }
  return %retVal;
}


sub slashFix{
  my $path = shift;
  if( $^O =~ /mswin/i){
    $path =~ s/\//\\/g;
  }
  else{
    $path =~ s/\\/\//g;
  }
  return $path;
}



1;

# Local Variables: ***
# perl-dbg-flags:"-w      -I../../../../../../wrk/script -I../../../sdbrun  " ***
# sdbinst-defines: "-define SCRIPTING=YES" ***
# sdbinst-testfile: "PropertiesTest.pl" ***
# sdbinst-script-directory:  "sys/wrk/script/SAPDB/Install/Templates" ***
# End: ***
