package SAPDB::Install::Templates2::InstTemplate;
use strict;
use SAPDB::Install::Templates2::Properties;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
use SAPDB::Install::Templates2::InstUnit;
use SAPDB::Install::Registry;
use SAPDB::Install::System;

 our @ISA;
 if($SCRIPTING){
   require Exporter;
   push @ISA,qw(Exporter);
 }
 else{
   require SDB::Common::Exporter;
   push @ISA,qw(SDB::Common::Exporter);
 }
push  @ISA, qw( SAPDB::Install::Templates2::Properties);

our @EXPORT = qw(createTemplateObject );


my $ISUNIX = $^O =~ /mswin/i ? "0" : "1";
my $ISWINDOWS = $^O =~ /mswin/i ? "1" : "0";


#ATTRIBUTES:
my $myData = << "XMLDATA";
<ParameterDefinitions>
  <Parameter id="packages" />
  <Parameter id="removeList" />
  <Parameter id="dBAUser" />
  <Parameter id="dBMUser" />
  <Parameter id="lCAUser" />
  <Parameter id="lCAEncoding" />
  <Parameter id="archiveDir" >
    <Type>PATH</Type>
  </Parameter>
  <Parameter id="indepData" >
    <Type>PATH</Type>
    <Default>\${APPLICATIONDATAPATH}/sdb/data</Default>
  <Description>path for release independend data</Description>
  </Parameter>
  <Parameter id="sWDrive" >
    <Type>DRIVE</Type>
    <Default>C</Default>
    <Description>Disk drive for software installation</Description>
  </Parameter>
  <Parameter id="indepProg"  >
    <Type>PATH</Type>
    <Default>\${SYSTEMPROGPATH}/sdb/programs</Default>
  <Description>Installation path for release independend software</Description>
  </Parameter>
  <Parameter id="softwarePath"  >
    <Type>PATH</Type>
    <Default>\${SYSTEMPROGPATH}/sdb</Default>
  <Description>Installation path for release independend software</Description>
  </Parameter>
  <Parameter id="systemDrive"  >
    <Type>DRIVE</Type>
   <Description>System Drive(Windows only)</Description>
  </Parameter>
  <Parameter id="registryFile" >
    <Type>PATH</Type>
  </Parameter>
  <Parameter id="ownerName" >
    <Type>STRING</Type>
    <Default>sdb</Default>
  </Parameter>
  <Parameter id="ownerGroup" >
    <Type>STRING</Type>
    <Default>sdba</Default>
  </Parameter>
  <Parameter id="systemProgPath" >
    <Type>PATH</Type>
  </Parameter>
  <Parameter id="applicationDataPath" >
    <Type>PATH</Type>
  </Parameter>
  <Parameter id="instance" />
  <Parameter id="upgradeFile" >
    <Type>PATH</Type>
  </Parameter>
  <Parameter id="logFile" >
    <Type>PATH</Type>
  </Parameter>
  <Parameter id="templateID" />
  <Parameter id="dBInstance" />
  <Parameter id="isIndepProgRegistered" />
  <Parameter id="isIndepDataRegistered" />
</ParameterDefinitions>
<ParameterData>
DBAUSER dba,dba
DBMUSER DBM,DBM
LCAUSER lcaadm,sap
LCAENCODING ascii
ARCHIVEDIR .
SOFTWAREPATH  \${SYSTEMPROGPATH}/sdb
REGISTRYFILE  \${INDEPDATA}/config/install/INSTREG
UPGRADEFILE upgrade.config
</ParameterData>
XMLDATA



my $xmlDataRaw = << "XMLDATA";
<Template id="desktop" instanceType="SAPDB::Install::Templates2::InstanceDesktop" type="SAPDB::Install::Templates2::DesktopTemplate">
 <Wizard  purpose="InstanceCreationWin" id="InstanceCreationWin_w1"/>
 <Wizard  purpose="InstanceCreationUx" id="InstanceCreationUx_w1"/>
 <Wizard  purpose="Overview"  id="Overview_w2"/>
</Template>
<Template id="custom" instanceType="SAPDB::Install::Templates2::InstanceCustom"  type="SAPDB::Install::Templates2::CustomTemplate">
 <Wizard  purpose="InstanceCreationWin"  id="InstanceCreationWin_w2" />
 <Wizard  purpose="InstanceCreationUx"  id="InstanceCreationUx_w2" />
 <Wizard  purpose="Overview"  id="Overview_w2" />
</Template>
<Template id="sapdw" instanceType="SAPDB::Install::Templates2::InstanceSAPDW" type="SAPDB::Install::Templates2::SAPDWTemplate">
 <Wizard  purpose="InstanceCreationWin"  id="InstanceCreationWin_w2" />
 <Wizard  purpose="InstanceCreationUx"  id="InstanceCreationUx_w2" />
 <Wizard  purpose="Overview"  id="Overview_w2" />
</Template>
<Template id="sapmc" instanceType="SAPDB::Install::Templates2::InstanceMobileClientEmpty" type="SAPDB::Install::Templates2::MobileClientTemplate">
 <Wizard  purpose="InstanceCreationWin"  id="InstanceCreationWin_w2" />
 <Wizard  purpose="InstanceCreationUx"  id="InstanceCreationUx_w2" />
 <Wizard  purpose="Overview"  id="Overview_w2" />
</Template>
<Template id="sapsbs" instanceType="SAPDB::Install::Templates2::InstanceSAPSBS" type="SAPDB::Install::Templates2::MobileClientTemplate">
 <Wizard  purpose="InstanceCreationWin"  id="InstanceCreationWin_w2" />
 <Wizard  purpose="InstanceCreationUx"  id="InstanceCreationUx_w2" />
 <Wizard  purpose="Overview"  id="Overview_w2" />
</Template>
<Template id="sapmds" instanceType="SAPDB::Install::Templates2::InstanceMobileClientArchive" type="SAPDB::Install::Templates2::MobileClientTemplateReuse"/>


<WizardDefinitions>
 <Wizard  id="InstanceCreationWin_w1" >
   <Frame previous=""                    id="DBNamePasswdDesktop" next="VolumesMinimal"/>
   <Frame previous="DBNamePasswdDesktop" id="VolumesMinimal"      next=""/>
 </Wizard>
 <Wizard  id="InstanceCreationWin_w2"  >
   <Frame previous=""                   id="DBNamePasswdSimple" next="VolumesAdvanced"/>
   <Frame previous="DBNamePasswdSimple" id="VolumesAdvanced"    next="SizeMemoryAdvanced"/>
   <Frame previous="VolumesAdvanced"    id="SizeMemoryAdvanced" next="GlobalDBSettings"/>
   <Frame previous="SizeMemoryAdvanced" id="GlobalDBSettings"   next=""/>
 </Wizard>
 <Wizard  id="InstanceCreationUx_w1" >
   <Frame previous=""                    id="DBNamePasswdDesktop" next="VolumesMinimalUx"/>
   <Frame previous="DBNamePasswdDesktop" id="VolumesMinimalUx"    next=""/>
 </Wizard>
 <Wizard  id="InstanceCreationUx_w2"  >
   <Frame previous=""                   id="DBNamePasswdSimple" next="VolumesAdvanced"/>
   <Frame previous="DBNamePasswdSimple" id="VolumesAdvanced"    next="SizeMemoryAdvanced"/>
   <Frame previous="VolumesAdvanced"    id="SizeMemoryAdvanced" next="GlobalDBSettings"/>
   <Frame previous="SizeMemoryAdvanced" id="GlobalDBSettings"   next=""/>
 </Wizard>
 <Wizard  id="Overview_w1" >
   <Frame previous=""                                 id="GeneralPropertiesDesktopOverview" next="SystemResOverview"/>
   <Frame previous="GeneralPropertiesDesktopOverview" id="SystemResOverview"                next=""/>
 </Wizard>
 <Wizard  id="Overview_w2" >
   <Frame previous=""                                 id="GeneralPropertiesCustomOverview"    next="GlobalDBSettingsOverview"/>
   <Frame previous="GeneralPropertiesCustomOverview"  id="GlobalDBSettingsOverview"           next="SystemResOverview"/>
   <Frame previous="GlobalDBSettingsOverview"         id="SystemResOverview"                  next=""/>
 </Wizard>
<WizardDefinitions>


  <FrameDefinitions>
   <Frame id="GeneralPropertiesCustomOverview"  >
    <Title>General Properties</Title>
     <ControlSequence>
       <Control parameter="db:dBName"  controlType="stringType" priority="0" Disabled="1">
          <Label>Database Name</Label>
       </Control>
       <Control parameter="db:dBMUser"  priority="1" controlType="stringType"  Disabled="1">
          <Label>Database Manager Operator</Label>
       </Control>
       <Control parameter="db:dBAUser"  priority="2" controlType="stringType"  Disabled="1">
          <Label>Database System Administrator</Label>
       </Control>
       <Control parameter="string:MONA (Password RED)"  priority="3" controlType="stringType"  Disabled="1" condition="db:loadSampleSchema">
          <Label>Tutorial User</Label>
       </Control>
      <Control parameter="db:array:dataVolumes"  priority="4" Disabled="1" controlType="pathType" Disabled="1">
        <Label>Data volume ##</Label>
      </Control>
      <Control parameter="db:array:logVolumes"  priority="5" Disabled="1" controlType="pathType" Disabled="1">
        <Label>Log volume ##</Label>
      </Control>
     </ControlSequence>
   </Frame>
   <Frame id="GeneralPropertiesDesktopOverview"  >
    <Title>General Properties</Title>
     <ControlSequence>
       <Control parameter="db:dBName"  controlType="stringType" priority="0" Disabled="1">
          <Label>Database Name</Label>
       </Control>
       <Control parameter="db:dBMUser"  priority="1" controlType="stringType"  Disabled="1">
          <Label>Database Manager Operator</Label>
       </Control>
       <Control parameter="db:dBAUser"  priority="2" controlType="stringType"  Disabled="1">
          <Label>Database System Administrator</Label>
       </Control>
       <Control parameter="string:MONA (Password RED)"  priority="3" controlType="stringType"  Disabled="1" condition="db:loadSampleSchema">
          <Label>Tutorial User</Label>
       </Control>
      <Control parameter="db:dBSize"  priority="4" Disabled="1" controlType="pagesType">
        <Label>Database Size</Label>
      </Control>
       <Control parameter="db:dataPath"  controlType="pathType" priority="5" >
          <Label>Data Area</Label>
       </Control>
       <Control parameter="db:logPath"  controlType="pathType" priority="6" >
          <Label>Log Area</Label>
       </Control>
     </ControlSequence>
   </Frame>
   <Frame id="SystemResOverview"  >
    <Title>System Resources</Title>
     <ControlSequence>
       <Control parameter="db:dBMemory"  controlType="stringType" priority="0" Disabled="1">
          <Label>Memory</Label>
       </Control>
       <Control parameter="db:cPUCount"  priority="1" controlType="stringType"  Disabled="1">
          <Label>CPUs</Label>
       </Control>
     </ControlSequence>
   </Frame>

   <Frame id="GlobalDBSettingsOverview">
    <Title>Global Settings</Title>
     <ControlSequence>
       <Control parameter="db:autoUpdateStatistics"  controlType="booleanType"  priority="0" Disabled="1">
          <Label>Automatic Statistics Update</Label>
       </Control>
       <Control parameter="db:autoRestart"           controlType="booleanType"  priority="1" Disabled="1">
          <Label>Automatic Start on System Reboot</Label>
       </Control>
       <Control parameter="db:logStrategyString"           controlType="stringType"   priority="2" Disabled="1">
          <Label>Log Settings</Label>
       </Control>
       <Control parameter="db:initialBackupFile"     controlType="pathType"     priority="3" Disabled="1" condition="isAutoLogStrategy">
          <Label>Initial Data Backup</Label>
       </Control>
       <Control parameter="db:logSaveDir"            controlType="pathType"     priority="4" Disabled="1" condition="isAutoLogStrategy">
          <Label>Log Backup Directory</Label>
       </Control>
     </ControlSequence>
   </Frame>



   <Frame id="DBNamePasswdSimple"  >
    <Title>Specify installation prefix and provide database authorization data</Title>
     <ControlSequence>
       <Control parameter="db:loadSampleSchema"  priority="2" controlType="booleanType" >
          <Label>Load Tutorial Data</Label>
       </Control>
       <Control parameter="db:adminUserStruct"  priority="3" controlType="adminusersType" >
          <Label>Database Users</Label>
       </Control>
       <Control parameter="db:dBName"  controlType="stringType" priority="1" >
          <Label>Database Name</Label>
       </Control>
       <Control parameter="softwarePath"  controlType="pathType" priority="0"  >
          <Label>Installation Prefix</Label>
       </Control>
     </ControlSequence>
   </Frame>
   <Frame id="VolumesAdvanced"  >
    <Title>Specify locations of database volumes</Title>
     <ControlSequence>
       <Control parameter="db:volstructLog"  controlType="logvolumesType" priority="2" >
          <Label>Log Volumes</Label>
       </Control>
       <Control parameter="db:volstructData"  controlType="datavolumesType" priority="2" >
          <Label>Data Volumes</Label>
       </Control>
     </ControlSequence>
   </Frame>
   <Frame id="SizeMemoryAdvanced"  >
    <Title>Specify database size and assign system resources</Title>
    <ControlSequence >
      <Control parameter="db:dBSize"  priority="0" Disabled="1" controlType="pagesType">
        <Label>Database Size</Label>
      </Control>
      <Control parameter="db:dBMemory"  priority="1"  controlType="sliderType" >
        <Label>Memory</Label>
      </Control>
      <Control parameter="db:cPUCount"  priority="2"  controlType="spinnerType" >
        <Label>CPUs</Label>
      </Control>
      <Control parameter="db:parallelConnections"  priority="3"  controlType="spinnerType" >
        <Label>Parallel Database Sessions</Label>
      </Control>
    </ControlSequence>
   </Frame>
   <Frame id="GlobalDBSettings"  >
     <Title>Specify global database settings</Title>
      <ControlSequence>
        <Control parameter="db:autoRestart"  priority="1"  controlType="booleanType" Disabled="$ISUNIX">
         <Label>Automatic Start On System Reboot</Label>
        </Control>
        <Control parameter="db:autoExtend"  priority="2"  controlType="booleanType" >
         <Label>Automatic Extension of Data Areas</Label>
        </Control>
        <Control parameter="db:loggingStruct"  priority="4"  controlType="loggingType">
         <Label>Log Settings</Label>
        </Control>
        <Control parameter="db:autoUpdateStatistics"  priority="3"  controlType="booleanType">
         <Label>Automatic Statistics Update</Label>
        </Control>
     </ControlSequence>
   </Frame>




   <Frame id="DBNamePasswdDesktop"  >
    <Title>Specify database name and provide authorization data</Title>
    <ControlSequence>
       <Control parameter="db:dBName"  controlType="stringType" priority="0" Disabled="">
          <Label>Database Name</Label>
       </Control>
       <Control parameter="db:dBSize"  priority="1" Disabled="0" controlType="pagesType">
          <Label>Database Size</Label>
       </Control>
       <Control parameter="db:loadSampleSchema"  priority="2" controlType="booleanType" >
          <Label>Load Tutorial Data</Label>
          <Description>Load Tutorial Data</Description>
          <LongDescription>Load Tutorial Data</LongDescription>
       </Control>
       <Control parameter="db:dBUser"  priority="3" controlType="dbuserType">
          <Label></Label>
          <Description></Description>
          <LongDescription></LongDescription>
       </Control>
    </ControlSequence>
   </Frame>

   <Frame id="VolumesMinimal"  >
    <Title>Specify disk drives for database software and database areas</Title>
    <ControlSequence>
       <Control parameter="sWDrive"  priority="0"  controlType="driveType" >
          <Label>Database Software</Label>
          <Description></Description>
       </Control>
       <Control parameter="db:volStructSimple"  priority="1" controlType="simplevolumesType" >
          <Label>Database Areas</Label>
          <Description>Database Areas</Description>
       </Control>
    </ControlSequence>
   </Frame>

   <Frame id="VolumesMinimalUx"  >
    <Title>Specify locations of database volumes</Title>
    <ControlSequence>
       <Control parameter="indepProg"  priority="0"  controlType="pathType">
          <Label>Database Software</Label>
          <Description></Description>
       </Control>
       <Control parameter="db:volStructSimple"  priority="1" controlType="simplevolumesType" >
          <Label>Database Areas</Label>
          <Description>Database Areas</Description>
       </Control>
    </ControlSequence>
   </Frame>


</FrameDefinitions>
XMLDATA

my @myXmlData = split(/\n/,$myData);
my @xmlData = split(/\n/,$xmlDataRaw);

#OVERWRITTEN METHODS:
sub GetXmlData{
  my $this = shift;
  my $data = $this->SUPER::GetXmlData();
  my @newData = @$data;
  push @newData,(@myXmlData);
  return \@newData;
}

sub parseLine{
  my ($this,$aLine) = @_;
  my $section = $this->{'section'};
  #Software Section:
  if ($section and $section =~ /Database Software/i) {
    $this->parseSoftwareParameters($aLine);
  }
  # Database Instance Section:
  elsif ($section and $section =~ /Database Instance/i) {
    my $db = $this->getDBInstance();
    $db->parseLine($aLine);
  }
}#parseLine;


sub Validate{
  my ($this) = @_;
  my $retVal;
  $retVal = $this->SUPER::Validate() and return $retVal;
  my $db = $this->getDBInstance();
  if($db){
    $retVal = $db->Validate() and return $retVal;
  }
  return 0;
}


sub parseSoftwareParameters{
  my ($this,$aLine) = @_;
  $this->SUPER::parseLine($aLine);
}




#INSTANCE METHODS:
sub new{
  my ($file,$id) = @_;
  my $this = {};
  blessme($this,$id);
  $this->SUPER::initData($file);
  $this->setTemplateID($id);
  return  $this;
}

sub blessme{
  my ($ar,$id) = @_;
  if($id){
    my @red = filterXMLTag(\@xmlData,'Template',"id=\"$id\"");
    my $retVal = parseXML
      (\@red,$ar,
       { 'beginTemplate'=> sub {
           bless $ar,$_[1]->{'type'};
         }
       });
  }
  else{
    bless $ar;
  }
}


sub createTemplateObject{
  my ($lines) = @_;
  my $anId;
  my @lines = split(/\n/,$lines);
  foreach(@lines){
    /^\s*TEMPLATE\s+(\w+)\s*$/ and $anId=$1
  }
  my $tmpl = new(undef,$anId);
  $tmpl->parseString($lines);
  return $tmpl;
}





sub GetConfigParameters{
  my ($this,$txt) = @_;
  $$txt = '';
  my $db = $this->getDBInstance();
  $$txt .= "# Configuration file for template '". $this->getTemplateID()."'\n";
  $$txt .= "# Generated at ". dateString() ." by MaxDB Installation Manager\n\n";
  $$txt .= "TEMPLATE " . $this->getTemplateID() . "\n\n";
  $$txt .= "[Database Software]\n";
  $$txt .= $this->GetConfigurationLines();
  if($db){
    $$txt .= "[Database Instance]\n";
    $$txt .= $db->GetConfigurationLines();
  }
  return 0;
}








sub nextFrame{
  my ($this,$frame,$wizard) = @_;
  unless($wizard){
    $wizard = 'InstanceCreation';
    my $osKenner = $^O =~ /mswin/i ? 'Win' : 'Ux';
    $wizard = $wizard . $osKenner;
  }
  my $retVal = 0;
  my $templateId = $this->getTemplateID();
  my $frameid ='';
  $frame  and $frameid = $frame->{'FrameId'};

  #Filter out the XML-Definition of  this instance:
  my @red = filterXMLTag(\@xmlData,'Template', "id=\"$templateId\"");

  #Filter out the reference to the wizard
  @red = filterXMLTag(\@red,'Wizard', "purpose=\"$wizard\"");

  #parse the reference to to the wizard to find the wizard id:
  my $wizardid;
  $retVal = parseXML(\@red,$this,
           { 'beginWizard'=>
             sub { my  ($this,$atts,$stack) = @_;
                   $wizardid = $atts->{'id'};}
           });
  #Filter out all wizard definitions
  @red = filterXMLTag(\@xmlData,'WizardDefinitions');
  #Filter  out the wizard definition with the wizard id:
  @red = filterXMLTag(\@red,'Wizard', "id=\"$wizardid\"");


  #Parse the wizard definition and find the next id 
  my $nextid;
  $retVal = parseXML(\@red,$this,
           { 'beginFrame'=>
             sub { my  ($this,$atts,$stack) = @_;
                   $atts->{'previous'} eq $frameid and $nextid = $atts->{'id'};}
           });

  $retVal and return Logit("Error getting next frame: ".$retVal,undef);

  #If there is no next frame, nextid is undef, return undef:
  $nextid or return undef;

  #Create the next frame and return ist:
  return $this->createFrame($nextid);
}#nextFrame

sub createTemplateOverView{
  my ($this) = @_;
  my @overView;
  my ($aFrame,$id);
  while($aFrame = $this->nextFrame($aFrame,'Overview')){
    push @overView, ($aFrame);
  }
  return \@overView;
}

sub createFrame{
  my($this,$id) = @_;
  $id or return undef;
  my @red = filterXMLTag(\@xmlData,'FrameDefinitions');
  @red = filterXMLTag(\@red,'Frame',"id=\"$id\"");
  my $frame = { 'FrameId' => $id, 'Data' => {} };
  my $txt = "";
  my $retVal;
  $retVal = parseXML(\@red,$this,
           { 'endTitle'=> sub { $frame->{'Title'}=$txt;$txt='';},
             'endDescription'=> sub { $frame->{'Description'}=$txt;$txt='';},
             'endLongDescription'=> sub { $frame->{'LongDescription'}=$txt;$txt='';},
             'cdata'=> sub { $txt.=$_[1];}
   }) and return Logit("Parse error creating frame: ".$retVal,undef);

  @red = filterXMLTag(\@red,'ControlSequence');
  my @controls;
  $txt='';
  my $control; 
  $retVal = parseXML(\@red,$this,
           { 'beginControl'=> sub {
               my ($this,$atts,$stack) = @_;
               $control = {};
               $control->{'parameter'} = $atts->{'parameter'};
               $control->{'priority'} = $atts->{'priority'};

               my $ds = $atts->{'Disabled'};
               my $condition = ($ds and $ds =~ /^\s*condition\s*=\s*(\w+)\s*$/) ? $1 : undef;

               $control->{'Disabled'}  = 
                 $condition ?  ($this->EvalCondition($condition)? 1 : 0) : $atts->{'Disabled'};
               exists $atts->{'condition'} and $control->{'condition'} = $atts->{'condition'};
               $control->{'id'} = $atts->{'controlType'};
             },
             'endControl'=> sub { push @controls, ($control); undef $control; },
             'endLabel'=> sub { $control->{'Label'}=$txt;$txt='';},
             'endDescription'=> sub { $control->{'Description'}=$txt;$txt='';},
             'endLongDescription'=> sub { $control->{'LongDescription'}=$txt;$txt='';},
             'cdata'=> sub { $txt.=$_[1];}
           })
    and return Logit("Parse error creating frame: ".$retVal,undef);
  foreach(@controls){
    my $ac = $_;
    $this->EvalCondition($ac->{'condition'}) and 
      $this->createTemplateParameter($ac,$frame);
  }
  return $frame;
}




sub EvalCondition{
  my ($this,$cond) = @_;
  $cond or return 1;
  my $prop = $this;
  if($cond =~ /^db:(.+)$/) {
    $prop = $this->getDBInstance();
    $cond = $1;
  }
  return $prop->Get($cond);
}

sub createTemplateParameter{
  my ($this,$ac,$frame) = @_;
  my $id = $ac->{'parameter'};
  my $param = $this->createParameterDescriptionById($id);
  if($id =~ /array:/){
    $this->dirtyParamControlInterSectionArray($ac,$param,$frame);
  }
  else{
    $this->dirtyParamControlInterSection($ac,$param,$frame);
  }
  return $param;
}

sub createParameterDescriptionById{
  my ($this,$id) = @_;
  my $param = undef;
  my $aId = $id;
  if($id =~ /^db:(.+)$/){
    $aId = $1;
    my $instance = $this->getDBInstance();
    $instance or return Logit("Cannot find instance object in InstTemplate::createTemplateParameter",undef);
    $param =  $instance->createParameterDescriptionById($aId);
  }
  else{
    $param =  $this->SUPER::createParameterDescriptionById($aId);
  }
  return $param;
}

sub dirtyParamControlInterSectionArray{
  my ($this,$ac,$params,$frame) = @_;
  my $counter=0;
  foreach my $param (@$params){
    $counter++;
    my %acc = %$ac;
    my $racc = \%acc;
    $racc->{'Label'} =~ s/##/#$counter/g;
    $racc->{'priority'} = $racc->{'priority'}. '.'.$counter;
    my $id = $racc->{'parameter'};
    my $countedId = $id.":".$counter;
    $racc->{'parameter'}=$countedId;
    $this->dirtyParamControlInterSection(\%acc,$param,$frame);
  }
  return $params;
}

sub dirtyParamControlInterSection{
  my ($this,$ac,$param,$frame) = @_;
  my $id = $ac->{'parameter'};
  $param->{'ControlType'}=$ac;
  $param->{'Description'}=$ac->{'Description'};
  $param->{'LongDescription'}=$ac->{'LongDescription'};
  $param->{'Label'}=$ac->{'Label'};

  $ac->{'unit'}=$param->{'unit'};
  my $deps = $param->{'dependencies'};
  if($deps){
    my @cdeps;
    my $prefix = $id =~ /^(\w+:)/ ? $1 : '';
    foreach(@$deps){
      push @cdeps,($prefix . $_);
    }
    $ac->{'dependencies'}=\@cdeps;
  }

  $frame and $frame->{'Data'}->{$id}=$param;
}

sub setAsyncCallback{
  my ($this,$subref) = @_;
  push @{$this->GetListeners('async_status')},($subref);
}
sub setStatusListCallback{
  my ($this,$subref) = @_;
  push @{$this->GetListeners('status_callback')},($subref);
}
sub setLogCallback{
  my ($this,$subref) = @_;
  push @{$this->GetListeners('log')},($subref);
  push @{$this->GetListeners('warning')},($subref);
  push @{$this->GetListeners('debug')},($subref);
}


sub ReadIndepProg{
  my ($aindep_data,$aindep_prog)=readIndepPath(); 
  my $retVal;
  if (defined $aindep_prog and not $aindep_prog =~ /^\s*$/) {
    $retVal = $aindep_prog;
  }
  return $retVal;
}

sub ReadIndepData{
  my ($aindep_data,$aindep_prog)=readIndepPath(); 
  my $retVal;
  if (defined $aindep_data and not $aindep_data =~ /^\s*$/) {
    $retVal = $aindep_data;
  }
  return $retVal;
}
#PARAMETERS:


#ARCHIVEDIR
sub initializeArchiveDir{
  my $archiveDir = $SADBP::Install::Config{'CallerDir'};
  $archiveDir or $archiveDir = '';
  $archiveDir = genAbsPath($archiveDir);
  $archiveDir =~ s/\/$//;
  return $archiveDir;
}


#SYSTEMDRIVE
sub defaultSystemDrive{
  my $sysProgPath = getSysProgPath();
  my ($drive)= ($sysProgPath =~ /^\s*(\w):.*/);
  return $drive;
}

#SOFTWAREPATH:
sub defaultSoftwarePath{
  return  catfile(getSysProgPath(),'sdb');
}

#SWDRIVE:
#This setter implicitly sets indepProg, indepData and the dependendPath of the instance
sub setSWDrive{
  my ($this,$newDrive)= @_;
  $newDrive =~ /^\s*(\w):?\s*$/ and $newDrive = $1;

  #call default setter:
  $this->Set('sWDrive',$newDrive,"no can");

  #if this is a new installation, set indepprog:
  unless($this->getIsIndepProgRegistered()){
    my $indepProg  = $this->getIndepProg();
    my $defaultIndepProg  = $this->defaultIndepProg();
    my ($indepProgDrive) = ($indepProg =~ /^\s*(\w):/);
    my ($defaultIndepProgDrive) = ($defaultIndepProg =~ /^\s*(\w):/);
    if(lc $newDrive eq lc $defaultIndepProgDrive){
      $this->setIndepProg($defaultIndepProg);
    }
    else{
      $this->setIndepProg(catfile("$newDrive:",'sdb','programs'));
    }
  }

  #if this is a new installation, set indepdata:
  unless($this->getIsIndepDataRegistered()){
    my $indepData  = $this->getIndepData();
    my $defaultIndepData  = $this->defaultIndepData();
    my ($indepDataDrive) = ($indepData =~ /^\s*(\w):/);
    my ($defaultIndepDataDrive) = ($defaultIndepData =~ /^\s*(\w):/);
    if(lc $newDrive eq lc $defaultIndepDataDrive){
      $this->setIndepData($defaultIndepData);
    }
    else{
      $this->setIndepData(catfile("$newDrive:",'sdb','data'));
    }
  }

  #if there is an instance, set the dependend path:
  my $instance = $this->getDBInstance();
  $instance or return;
  my $depPath = $instance->getDependendPath();
  my $sysProgPath = $this->defaultSystemProgPath();
  my ($depDrive) = ($depPath =~ /^\s*(\w):/);
  my ($sysDrive) = ($sysProgPath =~ /^\s*(\w):/);
  my $dbname = $instance->getDBName();
  if(lc $sysDrive eq lc $newDrive){
    $instance->setDependendPath(catfile($sysProgPath,'sdb',$dbname));
  }
  else{
    $instance->setDependendPath(catfile("$newDrive:",'sdb','programs',$dbname));
  }
}




sub validateSWDrive{
  my ($this,$val) = @_;
  $^O =~ /mswin/i or return 0;
  $val or return "Drive not specified!";
  if( $val =~ /^\s*(\w):?\s*$/){
    $val = $1;
  }
  else{
    return "Invalid drive '$val'";
  }
  $val or return "Invalid drive: <UNDEFINED>";
  $val =~ /^\w$/ or return "Invalid drive: '$val'";
  return 0;
}

#OWNERNAME:
sub initializeOwnerName{
  my $globs = readGlobals();
  my $val = $globs->{'SdbOwner'};
  $val or $val = 'sdb';
  return $val;
}

sub validateOwnerName{
  my ($this,$val) = @_;
  my $globs = readGlobals();
  my $registeredOwner = $globs->{'SdbOwner'};
  if($registeredOwner and $registeredOwner ne $val){
    return "DB owner must be $registeredOwner";
  }
  return 0;
}
#OWNERGROUP:
sub initializeOwnerGroup{
  my $globs = readGlobals();
  my $val = $globs->{'SdbGroup'};
  $val or $val = 'sdba';
  return $val;
}


sub validateOwnerGroup{
  my ($this,$val) = @_;
  my $globs = readGlobals();
  my $registeredGroup = $globs->{'SdbGroup'};
  if($registeredGroup and $registeredGroup ne $val){
    return "DB group must be $registeredGroup";
  }
  return 0;
}



#TEMPLATEID:

sub setTemplateID{
  my ($this,$id) = @_;
  #Set the template id:
  $this->Set('templateID',$id,"No can");
  $id or $this->setDBInstance(undef);
  $id or return;

  #side effect: initialize instance data:
  my $data = {'PACKAGE_NAME' =>'DBInstance[' . $id.']'};

  my @red = filterXMLTag(\@xmlData,'Template',"id=\"$id\"");
  my $retVal = parseXML
    (\@red,$this,
     { 'beginTemplate'=> sub {
         $data->{'CLASS'}=$_[1]->{'instanceType'};
       }
     }
    );
  $retVal and  return Logit("Parse error setting template id $id:".$retVal,undef);
  $data->{'CLASS'} or return Logit("Instance type not found for $id.",undef);
  my $instance =  SAPDB::Install::Templates2::InstUnit::new($data,$this);
  $instance or  Logit("Could not get create instance object for Template $id");
  $this->setDBInstance($instance);
}

#ISINDEPPROGREGISTERED
sub getIsIndepProgRegistered{
  return ReadIndepProg() ? 1 : 0;
}

#ISINDEPDATAREGISTERED
sub getIsIndepDataRegistered{
  return ReadIndepData() ? 1 : 0;
}
#SYSTEMPROGPATH:
sub defaultSystemProgPath{
  return getSysProgPath();
}
#APPLICATIONDATAPATH:
sub defaultApplicationDataPath{
  return getAppDataPath();
}


#INDEPDATA:
sub defaultIndepData{
  my  $retVal = catfile(getAppDataPath(),'sdb','data'); 
  return $retVal;
}
sub initializeIndepData{
  my $this = shift;
  my $retVal = $this->ReadIndepData();
  $retVal or $retVal = $this->GetDefault('indepData'); 
  return $retVal;
}

#INDEPPROG:
sub defaultIndepProg{
  my $retVal = catfile(getSysProgPath(),'sdb','programs');
  return $retVal;
}
sub initializeIndepProg{
  my $this = shift;
  my $retVal = $this->ReadIndepProg();
  $retVal or $retVal = $this->GetDefault('indepProg');
  return $retVal;
}



#PACKAGES:
sub parsePackages{
  my ($this,$key,$value ) = @_;
  my @packages = split/,/,$value;
  return \@packages;
}

sub formatPackages{
  my ($this,$key,$value ) = @_;
  my $packages = join ',',@$value;
  return (uc $key) . " " . $packages . "\n";
}

#REMOVELIST:
sub parseRemoveList{
  my ($this,$key,$value ) = @_;
  my @retVal;
  if ($value) {
    my @packages = split(/,/,$value);
    foreach (@packages) {
      if (/^(.*)\s*\[(.*)\]$/) {
        my $name = $1;
        my $path = $2;
        $name =~ s/\s+$//;
        $name =~ s/^\s+//;
        $path =~ s/\s+$//;
        $path =~ s/^\s+//;
        push @retVal,([$name,$path]);
      }
    }
  }
  return \@retVal;
}

sub getRemoveList{
  my $this = shift;
  my $l = $this->Get('removeList',"No can");
  my @outlist;
  foreach(@$l){
    my $name = $_->[0];
    my $path = $_->[1];
    $path = $this->recursiveSubstitute($path);
    push @outlist, ([$name,$path]);
  }
  return \@outlist;
}

sub formatRemoveList{
  my ($this,$key,$value ) = @_;
  my $retVal = '';
  foreach(@$value){
    $retVal .= ',' . $_->[0] . '[' . $_->[1] . ']';
  }
  $retVal =~ s/^,//;
  $retVal = (uc $key) . " " . $retVal. "\n";
  return $retVal;
}


#DBPARAM:
sub formatDBParam{
  my ($this,$key,$value ) = @_;
  my $retVal = "#Clearing inherited parameters (Do not delete this line):\nDBPARAM  \n";
  if($value){
    foreach(keys %$value){
      $retVal .=  'DBPARAM ' . $_ . ' ' . $value->{$_}  . "\n";
    }
  }
  return  $retVal;
}

#DBVOLUMES:
sub formatDBVolumes{
  my ($this,$key,$value ) = @_;
  my $retVal = "#Clearing inherited volumes (Do not delete the following  line):\nDBPARAM  \n";
  if($value){
    foreach(@$value){
      $retVal .=  'DBVOLUME '. $_ . "\n";
    }
  }
  return  $retVal;
}
sub getDBVolumes{
  my $this = shift;
  my $l = $this->Get('dBVolumes',"no can","no subst");
  my @outlist;
  foreach(@$l){
    my $val = $this->recursiveSubstitute($_);
    push @outlist, ($val);
  }
  return \@outlist;
}


package SAPDB::Install::Templates2::CustomTemplate;
use strict;
use SAPDB::Install::Templates2::Properties;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;

 our @ISA;
 if($SCRIPTING){
   require Exporter;
   push @ISA,qw(Exporter);
 }
 else{
   require SDB::Common::Exporter;
   push @ISA,qw(SDB::Common::Exporter);
 }
push  @ISA, qw( SAPDB::Install::Templates2::InstTemplate);

my $customData = << "CUSTOMDATA";
<ParameterDefinitions>
  <Parameter id="softwarePath" persistent="YES"/>
</ParameterDefinitions>
CUSTOMDATA
my @customData = split(/\n/,$customData);

#OVERWRITTEN METHODS:
sub GetXmlData{
  my $this = shift;
  my $data = $this->SUPER::GetXmlData();
  my @newData = @$data;
  push @newData,(@customData);
  return \@newData;
}


















package SAPDB::Install::Templates2::DesktopTemplate;
use strict;
use SAPDB::Install::Templates2::Properties;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;

 our @ISA;
 if($SCRIPTING){
   require Exporter;
   push @ISA,qw(Exporter);
 }
 else{
   require SDB::Common::Exporter;
   push @ISA,qw(SDB::Common::Exporter);
 }
push  @ISA, qw( SAPDB::Install::Templates2::InstTemplate);


my $desktopData = << "DESKTOPDATA";
<ParameterDefinitions>
  <Parameter id="sWDrive" persistent="YES:win"/>
  <Parameter id="indepProg" persistent="YES:ux"/>
</ParameterDefinitions>
DESKTOPDATA
my @desktopData = split(/\n/,$desktopData);

#OVERWRITTEN METHODS:
sub GetXmlData{
  my $this = shift;
  my $data = $this->SUPER::GetXmlData();
  my @newData = @$data;
  push @newData,(@desktopData);
  return \@newData;
}

package SAPDB::Install::Templates2::SapServerTemplate;
use strict;
use SAPDB::Install::Templates2::Properties;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;

 our @ISA;
 if($SCRIPTING){
   require Exporter;
   push @ISA,qw(Exporter);
 }
 else{
   require SDB::Common::Exporter;
   push @ISA,qw(SDB::Common::Exporter);
 }
push  @ISA, qw( SAPDB::Install::Templates2::InstTemplate);


my $sapServerData = << "SAPSERVERDATA";
<ParameterDefinitions>
</ParameterDefinitions>
<ParameterData>
</ParameterData>
SAPSERVERDATA
my @sapServerData = split(/\n/,$sapServerData);

#OVERWRITTEN METHODS:
sub GetXmlData{
  my $this = shift;
  my $data = $this->SUPER::GetXmlData();
  my @newData = @$data;
  push @newData,(@sapServerData);
  return \@newData;
}



#PARAMETERS:

package SAPDB::Install::Templates2::SAPDWTemplate;
use strict;
use SAPDB::Install::Templates2::Properties;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
our @ISA =  qw( SAPDB::Install::Templates2::InstTemplate);
my $sapdwdata = << "SAPDWDATA";
<ParameterDefinitions>
  <Parameter id="sWDrive" persistent="YES:win"/>
  <Parameter id="softwarePath" persistent="YES:ux"/>
</ParameterDefinitions>
<ParameterData>
</ParameterData>
SAPDWDATA
#OVERWRITTEN METHODS:
sub GetXmlData{
  my $this = shift;
  $this->MergeData($sapdwdata,$this->SUPER::GetXmlData());
}
#PARAMETERS:
#SOFTWAREPATH:
sub defaultSoftwarePath{
  return  $^O =~ /mswin/i ? '${SWDRIVE}:/sapdb' : '/sapdb';
}
sub initializeSoftwarePath{defaultSoftwarePath();}
#INDEPPROG:
sub defaultIndepProg{ "\${SOFTWAREPATH}/programs"; }
#INDEPDATA:
sub defaultIndepData{ "\${SOFTWAREPATH}/data"; }

sub setSWDrive{
  my ($this,$newDrive)= @_;
  $newDrive =~ /^\s*(\w):?\s*$/ and $newDrive = $1;
  #call default setter:
  $this->Set('sWDrive',$newDrive,"no can");
}




package SAPDB::Install::Templates2::MobileClientTemplate;
use strict;
use SAPDB::Install::Templates2::Properties;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;

 our @ISA;
 if($SCRIPTING){
   require Exporter;
   push @ISA,qw(Exporter);
 }
 else{
   require SDB::Common::Exporter;
   push @ISA,qw(SDB::Common::Exporter);
 }
push  @ISA, qw( SAPDB::Install::Templates2::InstTemplate);


my $mobileClientData = << "MOBILECLIENTDATA";
<ParameterDefinitions>
  <Parameter id="indepProg">
  </Parameter>
  <Parameter id="indepData">
  </Parameter>
  <Parameter id="softwarePath"  persistent="YES:ux">
    <Type>PATH</Type>
  <Description>Root installation path</Description>
  </Parameter>
  <Parameter id="drive"  persistent="YES:win" >
    <Default>\${SYSTEMDRIVE}</Default>
    <Description>MaxDB Installation drive</Description>
  </Parameter>
  <Parameter id="sloppy"  >
    <Default>NO</Default>
    <Description>Allow different values for indep prog and indep data</Description>
  </Parameter>
</ParameterDefinitions>
<ParameterData>
</ParameterData>
MOBILECLIENTDATA
my @mobileClientData = split(/\n/,$mobileClientData);

#OVERWRITTEN METHODS:
sub GetXmlData{
  my $this = shift;
  my $data = $this->SUPER::GetXmlData();
  my @newData = @$data;
  push @newData,(@mobileClientData);
  return \@newData;
}


#PARAMETERS:

#DRIVE:
#This setter implicitly sets indepProg, indepData and the dependendPath of the instance
sub setDrive{
  my ($this,$newDrive)= @_;
#On windows, ignore this parameter:
  if($^O =~ /mswin/i){
    return;
  }

  $newDrive =~ /^\s*(\w):?\s*$/ and $newDrive = $1;
  #call default setter:
  $this->Set('drive',$newDrive,"no can");

  #if this is a new installation, set indepprog:
  unless($this->getIsIndepProgRegistered()){
    $this->setIndepProg(catfile($newDrive.':','sdb','programs'));
  }

  #if this is a new installation, set indepdata:
  unless($this->getIsIndepDataRegistered()){
    $this->setIndepData(catfile($newDrive.':','sdb','data'));
  }
  $this->setSoftwarePath(catfile($newDrive.':','sdb'));
}

#INDEPPROG
sub validateIndepProg{
  my ($this,$val) = @_;
  $val  or return "Independ Programs Path is not defined";
  $val =~ /\S/ or return "Independ Programs Path is not specified";
  my $sloppy = isTrue($this->getSloppy());
  my $indepProg = $this->getIndepProg();
  $sloppy and return 0;
  $val =~ /^\s*(\w:)?[\\\/]s(ap)?db[\\\/]programs\s*$/i or
    return "Changing INDEPPROG to '$val' is FORBIDDEN! Value must be '$indepProg'";
  return 0;
}
sub defaultIndepProg{
   return  "\${SOFTWAREPATH}/programs";
}

#INDEPDATA
sub validateIndepData{
  my ($this,$val) = @_;
  $val  or return "Independ Data Path is not defined";
  $val =~ /\S/ or return "Independ Data Path is not specified";
  my $sloppy = isTrue($this->getSloppy());
  $sloppy and return 0;
  my $indepData = $this->getIndepData();
  $val =~ /^\s*(\w:)?[\\\/]s(ap)?db[\\\/]data\s*$/i or
    return "Changing INDEPDATA to  '$val' is FORBIDDEN! Value must be '$indepData'";
  return 0;
}
sub defaultIndepData{
    return "\${SOFTWAREPATH}/data";
}

#SOFTWAREPATH:
sub defaultSoftwarePath{
  return  $^O =~ /mswin/i ? '${SWDRIVE}:/sdb' : '/sdb';
}


package SAPDB::Install::Templates2::MobileClientTemplateReuse;
use strict;
use SAPDB::Install::Templates2::Properties;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
our @ISA =  qw( SAPDB::Install::Templates2::MobileClientTemplate);
my $mobileClientReuseData = << "MOBILECLIENTREUSEDATA";
<ParameterDefinitions>
  <Parameter id="instanceArchive" persistent="NO">
    <Default>\${ARCHIVEDIR}/IASDB.TGZ</Default>
    <Type>PATH</Type>
    <Description>Full Path to compressed archive containing the initial backup</Description>
  </Parameter>
  <Parameter id="initialBackup" persistent="NO">
    <Default>initialbackup</Default>
    <Type>PATH</Type>
    <Description>Full path of the initial backup</Description>
  </Parameter>
  <Parameter id="instanceBackup" persistent="YES">
    <Default>\${ARCHIVEDIR}/mdsbackup</Default>
    <Type>PATH</Type>
    <Description>Full path of the load backup</Description>
  </Parameter>
  <Parameter id="instanceArchiveBackup" persistent="NO">
    <Default>Backup</Default>
    <Type>PATH</Type>
    <Description>Location of the backup in the archive</Description>
  </Parameter>
  <Parameter id="archiveDir"  persistent="NO">
    <Type>PATH</Type>
    <Description>Directory where installation packages are located</Description>
  </Parameter>
</ParameterDefinitions>
<ParameterData>
</ParameterData>
MOBILECLIENTREUSEDATA
my @mobileClientReuseData = split(/\n/,$mobileClientReuseData);
#OVERWRITTEN METHODS:
sub GetXmlData{
  my $this = shift;
  my $data = $this->SUPER::GetXmlData();
  my @newData = @$data;
  push @newData,(@mobileClientReuseData);
  return \@newData;
}


package SAPDB::Install::Templates2::TemplateList;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;


#ATTRIBUTES:
my $listData = << "LISTDATA";
<TemplateList>
  <TemplateSection label="General">
    <Template id="desktop" label="Desktop PC / Laptop" purpose="batch,gui">
      <Description>
       Typical installation for a single user environment. 
       Can be used during development of simple database applications, for 
       the MaxDB tutorial or for testing purposes.
      </Description>
    </Template>
    <Template id="custom" label="Custom" purpose="batch,gui">
      <Description>
       Customizable installation of a databases instance. 
       Default values for memory usage, cpu usage and other parameters are calculated and can be adapted.
       Any number of data volumes on multiple hard disks can be specified. 
      </Description>
    </Template>
  </TemplateSection>
  <TemplateSection label="SAP">
    <Template id="sapdw" label="SAP Developer Workplace"  purpose="batch,gui">
      <Description>
        SAP Developer Workplace
      </Description>
    </Template>
    <Template id="sapmc" label="SAP Mobile Client" purpose="batch,gui">
      <Description>
        Set up a empty database for mobile clients.
      </Description>
    </Template>
     <Template id="sapmds" label="SAP Mobile Client, mass device setup" purpose="batch">
       <Description>
         Use a backup to set up a mobile client instance
       </Description>
     </Template>
    <Template id="sapsbs" label="SAP SBS" purpose="batch,gui">
      <Description>
        Set up a database for SBS
      </Description>
    </Template>
  </TemplateSection>
</TemplateList>
LISTDATA


my ($templateList,$templateSection,$template,$description);
#CALLBACK FUNCTIONS FORM XML PARSER:
sub beginTemplateList{
  my  ($atts,$stack) = @_;
  $templateList = [];
}
sub endTemplateList{
  my  ($name,$stack) = @_;
}
sub beginTemplateSection{
  my  ($atts,$stack) = @_;
  $templateSection = {'label' => $atts->{'label'},'data'=>[]};
}
sub endTemplateSection{
  push @$templateList,($templateSection);
  undef $templateSection;
}
sub beginTemplate{
  my  ($atts,$stack) = @_;
  $template = {'TemplateCode'=>$atts->{'id'},
               'ShortDescription'=>$atts->{'label'},
               'purpose'=>$atts->{'purpose'}
              };
}
sub endTemplate{
  push @{$templateSection->{'data'}},($template);
  undef $template;
}
sub beginDescription{
  $description='';
}
sub endDescription{
  $template->{'LongDescription'}=$description;
  undef $description;
}
sub cdata{
  my ($line,$stack) = @_;
  defined $description or return;
  chomp $line;
  $line =~ s/^\s+//;
  $line =~ s/\s+$/ /;
  $description .= $line;
}



#INSTANCE METHODS:

sub GetTemplateDescr{
  my ($list,$id)=@_;
  foreach my $section (@$list) {
    my $label = $section->{'label'};
    my $data = $section->{'data'};
    foreach my $tmpl (@$data) {
      my $aid = $tmpl->{'TemplateCode'};
      $aid eq $id and return $tmpl;
    }
  }
  return undef;
}

sub GetTemplateDescrTxt{
  my ($list,$id)=@_;
  my $retVal = '';
  my $desc =  $list->GetTemplateDescr($id);
  $retVal .=  $desc->{'ShortDescription'}."\n";
#  my $tmpl = SAPDB::Install::Templates2::InstUnit::new(undef,$id);
  return $retVal;
}

#CONSTRUCTOR:
sub createTemplateList{
  my ($select) = @_;
  $select or $select = 'gui';
  my @lines = split(/\n/,$listData);
  my $retVal = parseXML(\@lines);
  $retVal and Logit("Error parsing template list: ".$retVal);
  filterTemplates($templateList,$select);
  bless $templateList;
  return $templateList;
}

sub createCompleteTemplateList{
  my ($select) = @_;
  return createTemplateList('gui,batch');
}
sub GetTemplateDescription{
  my ($id) = @_;
  my $list = createCompleteTemplateList();
  return GetTemplateDescrTxt($list,$id);
}


sub filterTemplates{
  my ($templateStruct,$sel) = @_;
  my @sel = split(/,/,$sel);
  foreach my $section (@$templateStruct) {
    my $label = $section->{"label"};
    my $templateDescriptions = $section->{"data"};
    my @tokeep;
    foreach my $templateDescription (@$templateDescriptions) {
      my $purpose = $templateDescription->{"purpose"};
      my $id = $templateDescription->{"TemplateCode"};
      my @purposes = split(/,/,$purpose);
      my $found;
      foreach my $ap (@purposes) {
        foreach my $as (@sel) {
          if ($ap eq $as) {
            $found = 1;
            last;
          }
        }
        $found and last;
      }
      $found and push @tokeep,($templateDescription);
    }
    $section->{"data"}=\@tokeep;
  }
}








1;




# Local Variables: ***
# eval:   (setq my-own  (sdb-get-current-own)) ***
# eval:   (setq my-sdbrun (format "%s\\sys\\wrk\\fast\\install\\tmp\\sdbrun" my-own)) ***
# eval:   (setq my-src (format "%s\\sys\\src\\install\\perl" my-own)) ***
# eval:   (setq perl-dbg-flags (format "-w -Ic:\\aw\\patch -I%s -I%s\\readkey -I%s" my-sdbrun my-sdbrun  my-src))***
# End: ***
