package SAPDB::Install::Templates2::test::PropertiesTest;
use SAPDB::Install::Templates2::Properties;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;
use strict;

require Exporter;
our @ISA = qw( Exporter SAPDB::Install::Templates2::Properties);

if($SCRIPTING){
  require Exporter;
  push @ISA,qw(Exporter);
  require Data::Dumper;
  import Data::Dumper;
}
else{
  require SDB::Common::Exporter;
  push @ISA,qw(SDB::Common::Exporter);
}



sub new(){
  my $this = {};
  bless $this ;
  $this->SUPER::initData();
  return  $this;
}
my $testData = << "TESTDATA";
<ParameterDefinitions>
  <Parameter id="packageName"  persistent="YES">
    <Description>This is the name of a package</Description>
  </Parameter>
  <Parameter id="packageDescription" dependencies="packageName,packageVersion" persistent="YES"/>
  <Parameter id="packageVersion" persistent="YES">
    <Validator>^\\d\\d\\.\\d\\d\\.\\d\\d\$</Validator>
    <Description>
This is the  multiline description of 
the package version. I has
three lines altogether
</Description>
  </Parameter >
  <Parameter id="packageSize" persistent="YES">
    <Type>DATASIZE</Type>
  </Parameter >
  <Parameter id="packagePath" persistent="YES">
    <Default>c:/Program Files</Default>
    <Description>Path of a package</Description>
  </Parameter>
</ParameterDefinitions>
<ParameterData>
<ParameterValue id="packageDescription">
  This is a multiline
  description of a package
</ParameterValue>
</ParameterData>
TESTDATA
my @testData = split(/\n/,$testData);

sub GetXmlData{ return \@testData; }


sub initializePackageName(){
  return "Initial name of a package";
}
sub validatePackagePath{
  return "";
}

sub validatePackageName{
  my ($this,$newVal,$key) = @_;
  my $printNewVal = defined $newVal ? $newVal : "<UNDEFINED>";
  return "";
}
sub parsePackagePath{
  my($this,$aKey,$aValue) = @_;
  return $aValue;
}


sub parsePackageDescription{
  my($this,$aKey,$aValue) = @_;
  return $aValue;
}
sub formatPackageDescription{
  my($this,$aKey,$aValue) = @_;
  return
    "<ParameterValue name=\"$aKey\">\n".
      $aValue.
        "</ParameterValue>\n" ;
}


sub formatPackagePath{
  my($this,$aKey,$aValue) = @_;
  return (uc $aKey) . " " .$aValue;
}



sub setPackageName(){
  my ($this ,$val) = @_;
  $this->Set('packageName',$val,"no can");
}

my ($props,$templateFile);
sub setUp{
  $props = new();
  my $retVal ; 
  $retVal = $props->parseFile($templateFile) and die $retVal;
}
sub tearDown{
  undef $props;
}


sub testGetPackageSize{
  my $v = $props->getPackageSize();
  $v == 128  or return "Wrong package size: $v";
  return 0;
}


sub testGetInitial{
  my $init = $props->GetInitial('packageName');
  $init or return "No initial for packageName";
  return 0;
}

sub testInitializePackageName(){
  my $packageName =   $props->getPackageName();
  $packageName or return "No value for packageName";
  return 0;
}

sub testDescribe{
  my $txt =   $props->Describe();
  return 0;
}

sub testExistsKey{
  my $s = $props->ExistsKey('useless');
  $s and return "Assertion failed: existsKey('useless')";
  $s = $props->ExistsKey('packageName');
  $s or return "Assertion failed: existsKey('packageName')";
  return 0;
}

sub testGetConfigurationLines(){
  my $txt =   $props->GetConfigurationLines();
  return 0;
}
sub testIsPersistent{
  $props->IsPersistent('packageDescription') or return "Assertion failed: ispersistent";
  return 0;
}



sub testGetDescription{
  my $descr = $props->GetDescription('packageVersion');
  $descr or return "No description";
  return 0;
}

sub testCan{
  $props->can('getPackagePath') and return "I can do it";
}

sub  testMergeKeys{
  my $newKeys = {'packageName'  =>
                 {
                  'VALIDATOR' => sub{
                    my ($this,$newVal,$key) = @_;
                    my $printNewVal = defined $newVal ? $newVal : "<UNDEFINED>";
                    return "";
                  },
                 },
                 'packageVersion' =>
                 {
                  'VALIDATOR' => "schrott",
                 } ,
                 'packagePath' =>
                 {
                  'DEFAULT' => "c:/Programme"
                 } 
                };
  $props->MergeKeys($newKeys);
  my $def =   $props->GetDefault('packagePath');
  $def eq "c:/Programme" or return "Wrong default '$def'";
  $def =   $props->GetValidator('packageVersion');
  $def eq "schrott" or return "Wrong validator '$def'";
  return 0;
}


sub testGetter{
  my $nm = $props->getPackagePath();
  $nm eq "c:/Program Files" or return "Wrong package path:'$nm'";
  return 0;
}

sub testSetter{
  $props->setPackagePath("testwert");
  my $val = $props->getPackagePath();
  "testwert" eq $val or return "Wrong package path:'$val'";
  $props->setPackageName("fsfsdfsdf");
  return 0;
}


sub testValidate{
   $props->Validate();
}

sub testValidateKey{
  $props->ValidateKey('testvalue','packagePath');
  return 0;
}

sub testGetValidator{
  $props->GetValidator('packagePath') or return "Validator not found!";
  return 0;
}

sub testGetFormatter{
  $props->GetFormatter('packagePath') or return "Formatter not found!";
  return 0;
}

sub testGetParser{
  $props->GetParser('packagePath') or return "Parser not found!";
  return 0;
}

sub testCreateParameterDescriptionById(){
  my $p = $props->createParameterDescriptionById('packageDescription');
  return 0;
  #  print Dumper($p);
}

sub testGetPackageDescription{
  my $desc = $props->getPackageDescription();
#  print "'$testData'\n";
  return 0;
}

my $testLines = << "LINES";
PACKAGEPATH c:/Documente und Einstellungen

LINES

sub testParseFile{
  my @array =   split(/\n/,$testLines);
  my $retVal =   $props->parseLineArray(\@array);
  my $path = $props->getPackagePath();
  $path eq 'c:/Documente und Einstellungen' or return "Wrong path: $path";
  return $retVal;
}
sub allTests{
  $templateFile = shift;
  unitTest(__PACKAGE__);
}



# Local Variables: ***
# eval:  (setenv "MYOWN" (sdb-get-current-own)) ***
# eval: (setq my-sdbrun (format "%s\\sys\\src\\install\\perl\\sdbrun" "c:\\SapDevelop\\aw")) ***
# eval: (setq my-script (format "%s\\sys\\wrk\\script" "c:\\SapDevelop\\aw")) ***
# eval: (setq my-src (format "%s\\sys\\src\\install\\perl" (sdb-get-current-own))) ***
# eval: (setq perl-dbg-flags (format "-w -I%s -I%s\\readkey -I%s -I%s" my-sdbrun my-sdbrun my-script my-src )) ***
# End: ***
