#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_27/sys/src/install/perl/SAPDB/Install/User/Config/Helpers.pm#1 $
# $DateTime: 2006/04/27 14:44:55 $
# $Change: 124736 $
#
# Desc: 

package SAPDB::Install::User::Config::Helpers;

use SDB::Common::Getopt::Long;
use SAPDB::Install::System;
use SAPDB::Install::StdIO;
use SAPDB::Install::Registry;

require SDB::Common::Exporter;

our @ISA = qw (SDB::Common::Exporter);
our @EXPORT = qw (check_dbname check_passwd get_java_path
				  get_java_version get_java check_port
				  get_syncman_configfile
				  gen_syncservice_accesskey
				  check_passwd_jms_adm);



sub gen_syncservice_accesskey{
	my ($dbname) = @_;
	if(check_dbname($dbname)){
		require SAPDB::Install::Hostname;
		my $hostname = SAPDB::Install::Hostname::hostname();
		$hostname =~ s/\..*//g;
		$hostname =~ tr/[a-z]/[A-Z]/;
		$dbname =~ tr/[a-z]/[A-Z]/;
		return 'SYNCSERVICE_'.$hostname.'_'.$dbname; 	 	
	}
	else{
		return undef;
	}
}


sub get_syncman_configfile{
	my $config_file;
	if($^O =~ /mswin/i){
		my $apps_path = getUsersAppDataPath;
		unless($apps_path =~ /\S/){
			print STDERR "cannot get users appdata path\n";
		}
		unless(-d $apps_path){
			print STDERR "users appdata path $apps_path doesn\'t exist\n";
		}
		$apps_path =~ s/\\/\//g;
		$config_file = $apps_path.'/sdb/syncman.cfg';
	}
	else{
		my $home = (getpwuid($<))[7];
		unless($home =~ /\S/){
			print STDERR "cannot get home directory\n";
		}
		unless(-d $home){
			print STDERR "home directory $home doesn\'t exist\n";
		}
		$config_file = $home.'/.sdb/syncman.cfg';
	 }
	return $config_file;
}


sub get_java_version{
	my ($java) = @_;
	my $version;
	if($^O =~ /mswin/i){
		$java =~ s/\//\\/g;
		$version = `\"$java\" -version 2>&1`;
	}
	else{
		$version = `$java -version 2>&1`;
	}	
	($version) = ($version =~ /\"(.*)\"/);
	return $version;
}

sub get_java{
	my $java;
	if(exists $ENV{'JAVA_HOME'} and -f $ENV{'JAVA_HOME'}.'/bin/java'.($^O =~/mswin/i ? '.exe' : '')){
		$java = $ENV{'JAVA_HOME'}.'/bin/java'.($^O =~ /mswin/i ? '.exe' : '');	
		my $version = get_java_version($java);
		if($version =~ /^1\.4\./){
			$java =~ s/\//\\/g if $^O =~ /mswin/i;
			return $java;
		}
	}
	my @javas = which('java');
	foreach my $test_java (@javas){
		my $version = get_java_version($test_java);
		if($version =~ /^1\.4\./){
			return $test_java;
		}
	}
	return undef;
}

sub get_java_path{
	my $java = get_java();
	return undef unless defined $java;
	my $java_path = $java;
	$java_path =~ s/[\\\/][^\\\/]+$//;
	return $java_path;
}

sub check_dbname{
	my ($dbname) = @_;
	if($dbname !~ /^[\w_]{1,8}$/){
		print STDERR "invalid database name: 1-8 alpha numerical characters allowed\n";
		return 0; 
	}
	return 1;
}

sub check_passwd{
	my ($passwd,$default) = @_;
	if($passwd !~ /\S/){
		print STDERR "empty password isn\'t allowed\n";
		return 0;
	}
	unless($passwd eq $default){
		print "confirm password: ";
		my $confirmed = readstdin(1);
		chop $confirmed if ($confirmed =~ /\r$/);
		unless($confirmed eq $passwd){
			print STDERR "password confirmation failed\n";
			return 0;
		}
	}
	return 1;
}

sub check_passwd_jms_adm{
	my ($passwd,$default) =@_;
	if(length ($passwd) < 8){
		print STDERR "admin password needs at least 8 characters\n";
		return 0;
	}
	return check_passwd ($passwd,$default);
}


sub check_port{
	my ($value) = @_;
	if($value =~ /\D/){
		print STDERR "numeric value required\n";
		return 0; 
	}
	if($value < 1000){
		print STDERR "port number has to be at least 1000\n";
		return 0; 
	}
	return 1;
}

1;