#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_27/sys/src/install/perl/SAPDB/Install/Window/DBDrop.pm#1 $
# $DateTime: 2006/04/27 14:44:55 $
# $Change: 124736 $
#
# Desc: Installation dialog - extract files and do register calls
#

package SAPDB::Install::Window::DBDrop;
use Wx qw (
	wxEXPAND wxTOP wxALIGN_CENTER wxALL wxHORIZONTAL
	wxLEFT wxVERTICAL wxSUNKEN_BORDER wxTE_PASSWORD
	wxALIGN_CENTER_VERTICAL wxALIGN_RIGHT wxBOTTOM
	wxICON_ERROR wxCENTRE wxRIGHT wxICON_QUESTION 
	wxYES_NO wxID_YES);

use Wx::Event qw (EVT_BUTTON EVT_TEXT);

our @ISA = qw(Wx::Panel);

use SAPDB::Install::DBMCmd;
use SAPDB::Install::Registry;
use SAPDB::Install::Tools;
use SAPDB::Install::System;
use SAPDB::Install::Resources;

our $img_keys = SAPDB::Install::Resources->new('RESOURCES.TGZ')->
						getImages('icon/keys.png')->{'icon/keys.png'};

our ($indep_data,$indep_prog) = readIndepPath();

#
#	$my_args = {
#				'dbname' => <database name>,
#				'log' => <a object of type SAPDB::Install::Log or with similar interface>,
#			};
#



sub new{
	my ($class,$my_args,@args) = @_;
	my $self = $class->SUPER::new(@args);
	$self->{'dbname'} = $my_args->{'dbname'};
	$self->{'log'} = $my_args->{'log'};
	my $bitmap = Wx::StaticBitmap->new($self,-1,Wx::Bitmap->new($img_keys));
	my $topsizer = Wx::BoxSizer->new(wxVERTICAL);
	my $static_box = Wx::StaticBox->new($self,-1,'Database System Administrator Login');
	my $boxsizer = Wx::StaticBoxSizer->new($static_box,wxHORIZONTAL);
	my $edit_sizer = Wx::GridSizer->new(2,2);
	$edit_sizer->SetVGap(10);
	$edit_sizer->SetHGap(10);
	my $label_dbm = Wx::StaticText->new($self,-1,'User Name',[0,0],[-1,-1]);
	my $label_dbm_passwd = Wx::StaticText->new($self,-1,'Password',[0,0],[-1,-1]);
	$self->{'edit_dbm'} = Wx::TextCtrl->new($self,-1,'',[0,0],[70,-1]);
	$self->{'edit_dbm_pass'} = Wx::TextCtrl->new($self,-1,'',[0,0],[70,-1],wxTE_PASSWORD);
	$self->{'button_drop'} = Wx::Button->new($self,-1,'&Drop Database');
	$self->{'button_cancel'} = Wx::Button->new($self,-1,'&Cancel');
	EVT_BUTTON($self,$self->{'button_cancel'}->GetId,\&OnCancel);
	EVT_BUTTON($self,$self->{'button_drop'}->GetId,\&OnDrop);
	EVT_TEXT($self,$self->{'edit_dbm'}->GetId,\&OnTextChange);
	EVT_TEXT($self,$self->{'edit_dbm_pass'}->GetId,\&OnTextChange);
	$self->{'button_drop'}->Enable(0);
	my $button_sizer = Wx::BoxSizer->new(wxHORIZONTAL);
	$edit_sizer->Add($label_dbm,0,wxALIGN_CENTER_VERTICAL | wxLEFT , 10);
	$edit_sizer->Add($self->{'edit_dbm'},0,wxALIGN_RIGHT);
	$button_sizer->Add($self->{'button_drop'},0);
	$button_sizer->Add($self->{'button_cancel'},0,wxLEFT,20);
	$edit_sizer->Add($label_dbm_passwd,0,wxALIGN_CENTER_VERTICAL | wxLEFT , 10);
	$edit_sizer->Add($self->{'edit_dbm_pass'},0,wxALIGN_RIGHT);
	$edit_sizer->CalcMin;
	$boxsizer->Add($bitmap,0);
	$boxsizer->Add($edit_sizer,0, wxTOP | wxRIGHT | wxBOTTOM , 20);
	$boxsizer->CalcMin;
	$topsizer->Add($boxsizer,1,wxEXPAND | wxALL, 10 );
	$topsizer->Add($button_sizer,0,((wxALL) & ~(wxTOP)) | wxALIGN_RIGHT, 10);	
	$self->Fit;
	$self->SetSizer($topsizer);
	$self->SetAutoLayout(1);
	$self->Layout();
	$self->Fit;
	return $self;
}

sub OnTextChange{
	my ($self,$evt) = @_;
	my $text = $self->FindWindow($evt->GetId)->GetValue;
	if($text =~ /\S/){
		if ($self->{'edit_dbm'}->GetValue =~ /\S/ and 
			$self->{'edit_dbm_pass'}->GetValue =~ /\S/){
			$self->{'button_drop'}->Enable(1);
		}
	
	}
	else{
		$self->{'button_drop'}->Enable(0);
	}
	
	$evt->Skip(1);
}


sub OnCancel{
	my ($self,$evt) = @_;
	$self->GetParent->EndModal(0);
	#$self->Close(1);
	
}


sub OnDrop{
	my ($self,$evt) = @_;
	unless($self->drop()){
		my $msgbox = Wx::MessageDialog->new($self,$self->{'last_err'},
					'Error', wxICON_ERROR | wxCENTRE); 
		$msgbox->ShowModal;
		$msgbox->Destroy;
		if(exists $self->{'logon_failed'}){
			my $msgbox = Wx::MessageDialog->new($self,'Do you want to drop database '.
						$self->{dbname}.' without database authorization?'.
						"\n\n!!!YOU WILL LOSE ALL DATA INSIDE ".$self->{'dbname'}."!!!",
						'Force Dropping Database?' , wxICON_QUESTION | wxYES_NO | wxCENTRE);					  
			if($msgbox->ShowModal() == wxID_YES){
				$self->{'log'}->SetMsg("MSG: ".$self->{dbname}.": force db drop!\n");
				require SAPDB::Install::GetParam;
				
				my ($indep_data,$indep_prog) = readIndepPath();
				
				my $upc = $indep_data.'/config/'.$self->{'dbname'}.'.upc';
				my $upc_backup = $upc.'.fdd';
				my $paramfile = $indep_data.'/config/'.$self->{'dbname'};
				my $paramfile_backup = $paramfile.'.fdd';
				#my $upc = $indep_data.'/config'.$self->{'dbname'};
				my $params = SAPDB::Install::GetParam::GetParam($self->{'dbname'});
				$self->{'log'}->SetMsg("MSG: ".$self->{dbname}.": type of paramfile is ".$params->{'type'}."\n");
				my $rundir = $params->{'data'}->{'RUNDIRECTORY'};
				$self->{'log'}->SetMsg("MSG: ".$self->{dbname}.": rundirectory is $rundir\n");
				my $x_stop = $indep_prog.'/bin/x_stop';
				if($^O =~ /mswin/i){
					$x_stop .= '.exe';
					#$x_stop =~ s/\//\\/g;
				}
				
				if(-x $x_stop){
					$x_stop = "\"$x_stop\"" if($^O =~ /mswin/i); 
					my $cmd = "$x_stop ".$self->{dbname};
					callsystem($cmd,sub {return 1},1);
				}
				
				$rundir =~ s/\\/\//g;
				my $dbm_upc = $rundir.'/dbm.upc'; 
				my $dbm_upc_backup = $dbm_upc.'.fdd';
				
				if(-f $upc){
					unlink($upc_backup) if -f $upc_backup; 
					if(rename($upc,$upc_backup)){
						$self->{'log'}->SetMsg("MSG: ".$self->{dbname}.": usr profile container deleted\n");	
					}
					else{
						$self->{'log'}->SetMsg("MSG: ".$self->{dbname}.": deleting usr profile container failed: $!\n");
					}
				}
				else{
					$self->{'log'}->SetMsg("MSG: ".$self->{dbname}.": usr profile container doesn\'t exist\n");
				}
				
				if(-f $dbm_upc){
					unlink($dbm_upc_backup) if -f $dbm_upc_backup;
					if(rename($dbm_upc,$dbm_upc_backup)){
						$self->{'log'}->SetMsg("MSG: ".$self->{dbname}.": dbm usr profile container deleted\n");	
					}
					else{
						$self->{'log'}->SetMsg("MSG: ".$self->{dbname}.": deleting usr profile container failed: $!\n");
					}
				}
				else{
					$self->{'log'}->SetMsg("MSG: ".$self->{dbname}.": dbm usr profile container doesn\'t exist\n");
				}
				
				if(-f $paramfile){
					unlink($paramfile_backup) if -f $paramfile_backup;
					if(rename($paramfile,$paramfile_backup)){
						$self->{'log'}->SetMsg("MSG: ".$self->{dbname}.": parameter file deleted\n");	
					}
					else{
						$self->{'log'}->SetMsg("MSG: ".$self->{dbname}.": deleting parameter file failed: $!\n");
					}
				}
				else{
					$self->{'log'}->SetMsg("MSG: ".$self->{dbname}.": parameter file doesn\'t exist\n");
				}
				delete $self->{'logon_failed'};
				unless($self->drop()){
					my $msg = $self->{'last_err'};
					if($self->{'logon_failed'}){
						$msg = 'Sorry, DBM server of '.$self->{dbname}.' doesn\'t support this feature.';
					}
					my $msgbox = Wx::MessageDialog->new($self,$msg,
					'Forced database drop failed', wxICON_ERROR | wxCENTRE); 
					$msgbox->ShowModal;
					$msgbox->Destroy;
					
					#
					#	rollback
					#
					if(rename($upc_backup,$upc)){
						$self->{'log'}->SetMsg("MSG: ".$self->{dbname}.": usr profile container restored\n");
					}
					if(rename($dbm_upc_backup,$dbm_upc)){
						$self->{'log'}->SetMsg("MSG: ".$self->{dbname}.": dbm usr profile container restored\n");
					}
					if(rename($paramfile_backup,$paramfile)){
						$self->{'log'}->SetMsg("MSG: ".$self->{dbname}.": parameter file restored\n");
					}
					return undef;
				}
				
				#
				#	delete backup files
				#				
				unlink($upc_backup) if -f $upc_backup; 
				unlink($dbm_upc_backup) if -f $dbm_upc_backup; 
				unlink($paramfile_backup) if -f $paramfile_backup; 
				return 1;
			}
		}
		return undef;
	}
	
}



sub getstate{
	my ($self) = @_;
	my $prog  = "$indep_prog/bin/x_cons";
	if( $^O =~ /mswin/i){
		$prog .= '.exe';
		$prog =~ s/\//\\/g;
	}
	
	unless(-f $prog){
		return undef;
	}

	my $cmd  = '"'.$prog.'" '.$self->{'dbname'}.' state';
	 
	my $db_state;
	callsystem($cmd,1,sub{
		my ($text) = @_;
		foreach my $line (split("\n",$text)){
			if($line =~ /state/i){
				($db_state) = ($line =~ /\s(\S+)\s*$/);				
				last;
			}
		}
		return 1;	
	});
	unless($db_state =~ /\S/){
		return undef;
	}
	
	return $db_state;		
	
}

sub connect{
	my ($self) = @_;
	
	my $info = xserverInfo();
	unless($info->{'is_running'}){
		
		if(-f "$indep_prog/bin/x_server"){
			callsystem("$indep_prog/bin/x_server start");		
		}
		elsif(-f "$indep_prog/bin/x_server.exe"){
			callsystem("\"$indep_prog/bin/x_server.exe\" start" );
		}
		else{
			$self->{'last_err'} = 'Communication Error - cannot find x_server executable';
			return undef;
		}
		$info = xserverInfo();
		unless($info->{'is_running'}){
			$self->{'last_err'}  = 'Communication Error - cannot start communication server';
			return undef;			
		}
	}
	
	unless (exists $self->{'dbm'}){
		$self->{'dbm'} = SAPDB::Install::DBMCmd->new('',$self->{'dbname'});
	}
	unless(defined $self->{'dbm'}){
			$self->{'last_err'} = 'Connect failed - cannot connet to '.$db.'@localhost';
			return undef;
	}
	
	my $value = $self->{'edit_dbm'}->GetValue().
				','.$self->{'edit_dbm_pass'}->GetValue();
	
	my $rc = $self->{'dbm'}->user_logon($self->{'edit_dbm'}->GetValue().
				','.$self->{'edit_dbm_pass'}->GetValue());		
	
	unless(defined $rc){
		$self->{'last_err'} = 'user logon failed: '."\n".
							  $self->{'dbm'}->lastmsg;
		$self->{'logon_failed'} = 1;
		return undef;
	}	
	$self->{'connected'} = 1;
	
	$self->{'log'}->SetMsg("MSG: connection to database ".$self->{'dbname'}." successfully established\n");
	
	my $state = $self->{'dbm'}->db_state;
	unless(defined $state){
		$self->{'last_err'} = 'cannot get database state: '."\n".
							  $self->{'dbm'}->lastmsg;
		return undef;
	}
	$state =~ tr/[A-Z]/[a-z]/;
	$self->{'log'}->SetMsg("MSG: database ".$self->{'dbname'}." is $state\n");	
	
	if($state =~ /offline/i){
		$self->{'stopped'} = 1;
	}

	return 1;
}


sub stop{
	my ($self) = @_;
	unless($self->{'connected'}){
		unless($self->connect()){
			return undef;
		}	
	}
	my $rc = $self->{'dbm'}->db_offline;
	unless(defined $rc){
		$self->{'last_err'} = 'cannot stop database'."\n".
								$self->{'dbm'}->lastmsg;		
		return undef;
	}
	$self->{'log'}->SetMsg("MSG: database ".$self->{'dbname'}." stopped\n");
	$self->{'stopped'} = 1;
	return 1;
				
}


sub drop{
	my ($self) = @_;
	my $busy = Wx::BusyCursor->new();
	unless($self->{'connected'}){
		unless($self->connect()){
			return undef;
		}	
	}
	unless($self->{'stopped'}){
		unless($self->stop()){
			return undef;
		}
	}
	unless($self->{'dbm'}->db_drop){
		$self->{'last_err'} = 'drop database failed'."\n".
								$self->{'dbm'}->lastmsg;					
		return undef;
	}
	$self->{'dropped'} = 1;
	$self->{'log'}->SetMsg("MSG: database ".$self->{'dbname'}." dropped\n");
	delete $self->{$dbm};
	delete $self->{'connected'};
	$self->GetParent->EndModal(1);				
	return 1;
}


sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	
	if(defined $self->{'edit_dbm'}){
		$self->{'edit_dbm'}->Destroy;
		delete  $self->{'edit_dbm'};
	}
	if(defined $self->{'edit_dbm_pass'}){
		$self->{'edit_dbm_pass'}->Destroy;
		delete  $self->{'edit_dbm_pass'};
	}
	if(defined $self->{'button_drop'}){
		$self->{'button_drop'}->Destroy;
		delete $self->{'button_drop'};
	}
	if(defined $self->{'button_cancel'}){
		$self->{'button_cancel'}->Destroy;
		delete $self->{'button_cancel'};
	}
	$self->SUPER::Destroy;
}


sub DESTROY{
	my ($self) = @_;
	print ">>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}



1;
