package SAPDB::Install::Window::Desktop;

use Wx::Event qw(EVT_BUTTON EVT_KEY_DOWN);

use Wx qw(wxADJUST_MINSIZE wxALIGN_CENTER
			wxWANTS_CHARS wxWHITE);

use SAPDB::Install::Control::Icon;
use SAPDB::Install::Resources;
use SAPDB::Install::Dialog::About;	

our @ISA = qw(Wx::Panel);

sub new{
	my($this ) = shift->SUPER::new(@_);
	$this->SetWindowStyleFlag(wxWANTS_CHARS);
	$this->SetBackgroundColour(wxWHITE);
	EVT_KEY_DOWN($this,\&OnKeyDown);
	return $this;
}

sub initDesk{
	my ($self,$icon_defs) = @_;
	my $rc = SAPDB::Install::Resources->new('RESOURCES.TGZ');
	my @icon_defs = @$icon_defs;
	my @needed_imgs;
	foreach my $def (@icon_defs){
		push @needed_imgs,$def->[1];
	} 
	my $imgs = $rc->getImages(@needed_imgs);
	my $id = 0;
	my $ratio = 4/3;	
	$self->{'icons'} = [];
	local *icons = $self->{'icons'};
	
	my $max_cols = sqrt(($#icon_defs  + 1) * $ratio);
	if(($max_cols - int($max_cols)) >= 0.5){
		$max_cols = int($max_cols) + 1;
	}
	else{
		$max_cols = int($max_cols);
	}
	my $max_rows = int(sqrt(($#icon_defs  + 1) * (1/$ratio)));
	if(($max_rows - int($max_rows)) >= 0.5){
		$max_rows = int($max_rows) + 1;
	}
	else{
		$max_rows = int($max_rows);
	}
	$self->{'sizer'} = Wx::GridSizer->new($max_rows,$max_cols);	

	foreach my $icondef (@icon_defs){
		my $icon = SAPDB::Install::Control::Icon->new($self,$id);
		$icon->setImg($imgs->{$icondef->[1]});
		$icon->setText($icondef->[0]);
		$icon->{'cmd_func'} = $icondef->[2];
		push @icons,$icon;
		$self->{'sizer'}->Add($icon,$id,wxALIGN_CENTER | wxADJUST_MINSIZE);	
		EVT_BUTTON($self,$id++,\&OnIconClick);
	}
	
	#if($^O =~ /mswin/i){
	#	my $frame_icon = Wx::Icon->new(32,32,8);
	#	$frame_icon->CopyFromBitmap(Wx::Bitmap->new($imgs->{'icon/insticon.png'}->Scale(32,32)));
	#	$self->SetIcon($frame_icon);
	#}
	$self->{'selected'} = 0;
	$self->{'icons'}->[0]->select;
	$self->SetAutoLayout(1);
	$self->SetSizer($self->{'sizer'});
}

sub OnIconClick{
	my ($self,$event) = @_;
	$self->{'icons'}->[$self->{'selected'}]->unselect;
	$self->{'icons'}->[$event->GetId]->select;
	$self->{'selected'} = $event->GetId;	
}

sub OnKeyDown{
	my ($self,$event) = @_;
	if(	$event->GetKeyCode == 9 || 
		$event->GetKeyCode == 318 ||
		$event->GetKeyCode == 316
		){
		my $next = $self->{'selected'};
		my @icons = @{$self->{'icons'}};
		if(($event->GetKeyCode == 9 && $event->ShiftDown) || $event->GetKeyCode == 316){
			$next--;
			$next = $#icons if $next < 0;
		}
		else{
			$next++;
			$next = 0 if $next > $#icons;
		}
		$self->{'icons'}->[$self->{'selected'}]->unselect;
		$self->{'icons'}->[$next]->select;
		$self->{'selected'} = $next;
	}
	elsif($event->GetKeyCode == 13){
		$self->{'icons'}->[$self->{'selected'}]->Call;
	}
	elsif($event->GetKeyCode == 65){
		SAPDB::Install::Dialog::About->new($self,-1);
	}

}

1;