package SAPDB::Install::Window::ExecPlan;

use Wx::Event qw(EVT_GRID_SELECT_CELL EVT_GRID_EDITOR_HIDDEN EVT_GRID_EDITOR_SHOWN EVT_SIZE);
use Wx qw(wxEXPAND wxVERTICAL wxSUNKEN_BORDER wxEVT_GRID_CELL_LEFT_DCLICK);
use Wx::Grid;

use SAPDB::Install::Resources;
use SAPDB::Install::Grid::CellStateIconRenderer;
use SAPDB::Install::Grid::CellProgressBarRenderer;

our @ISA = qw(Wx::Panel);

sub new{
	my($this ) = shift->SUPER::new(@_);
	$this->{'grid'} = Wx::Grid->new($this,-1);
	$this->{'grid'}->SetWindowStyleFlag(wxSUNKEN_BORDER);	
	$this->{'grid'}->EnableCellEditControl(1);
	$this->{'grid'}->EnableGridLines(0);
	$this->{'grid'}->SetColLabelSize(0);
	$this->{'grid'}->SetRowLabelSize(0);
	$this->{'grid'}->SetCellHighlightROPenWidth(0);
	$this->{'grid'}->EnableEditing(0);
	$this->{'grid'}->SetCellHighlightPenWidth(0);
	$this->{'grid'}->DisableDragGridSize;
	#EVT_GRID_SELECT_CELL($this->{'grid'},\&OnGridSelectCell);
	#EVT_GRID_EDITOR_HIDDEN($this->{'grid'},\&OnHideEditor);
	#EVT_GRID_EDITOR_SHOWN($this->{'grid'},\&OnShowEditor);
	my $sizer = Wx::BoxSizer->new(wxVERTICAL);
	$sizer->Add($this->{'grid'},1,wxEXPAND);
	$this->SetAutoLayout(1);
	$this->SetSizer($sizer);
	return $this;
}

sub OnHideEditor{
	my ($grid,$evt) = @_;
	$grid->EnableCellEditControl(1);	
	return 1;
}

sub OnShowEditor{
	my ($grid,$evt) = @_;
	#$grid->EnableCellEditControl(0);
	$evt->Skip(1);	
	return 1;
}

sub OnGridSelectCell{
	my ($grid,$evt) = @_;
	# do nothing
	#
	#$grid->SetCurrentCell($evt->GetRow,$evt->GetCol);
	#$grid->GetCellEditor($evt->GetRow,$evt->GetCol)->Create;
	#$grid->GetCellEditor($evt->GetRow,$evt->GetCol)->Show(1);
	$evt->Skip(1);
	return 1;
}


sub init{
	my ($self,$plan,$config) = @_;
	return if($#{@$plan} == -1 );
	$self->{'plan'} = $plan;
	delete $self->{'current_step'};
	delete $self->{'done'};
	$self->{'config'} = $config;
	my $cols = 2;
	if($config->{'with_progress'}){
		$cols = 3;
	}
	$self->{'grid'}->CreateGrid($#{@$plan} + 1 ,$cols);
	$self->{'grid'}->SetColMinimalWidth(0,50);
	if($config->{'with_progress'}){
		$self->{'grid'}->SetColMinimalWidth(2,200);
	}
	my $row = 0;
	#$self->{'grid'}->SetDefaultEditor(SAPDB::Install::Grid::CellProgressBarEditor->new);
	foreach my $step (@$plan){
		$self->{'grid'}->SetCellRenderer($row,0,SAPDB::Install::Grid::CellStateIconRenderer->new);
		if($config->{'with_progress'}){
			$self->{'grid'}->SetCellRenderer($row,2,SAPDB::Install::Grid::CellProgressBarRenderer->new);
			#$self->{'grid'}->SetCellEditor($row,2,SAPDB::Install::Grid::CellProgressBarEditor->new);
			#$self->{'grid'}->SetGridCursor($row,2);
			#$self->{'grid'}->EnableCellEditControl(1);
			#$self->{'grid'}->ShowCellEditControl;
	
		}
		$self->{'grid'}->SetCellValue($row,0,$step->{'Status'});
		$self->{'grid'}->SetCellValue($row++,1,$step->{'Label'});
	}
	#$self->{'grid'}->Fit;
	$self->{'grid'}->SetColSize(0,50);
	#$self->{'grid'}->SetColSize(1,250);
	$self->{'grid'}->AutoSizeColumn(1,1);
	if($config->{'with_progress'}){
		$self->{'grid'}->SetColSize(2,250);
		EVT_SIZE($self->{'grid'},sub {
			my ($grid,$evt) = @_;
			my $w = $grid->GetClientSize()->GetWidth() - 
				$grid->GetColSize(0) - 
				$grid->GetColSize(1);
			$grid->SetColSize(2,$w - 20);
		});
	}
	$self->Layout();
	return;
}


sub Next{
	my ($self,$state) = @_;
	return unless defined $self->{'plan'};
	return if $self->{'done'};
	unless(exists $self->{'current_step'}){
		$self->{'current_step'} = 0;	
		$self->{'grid'}->SetCellValue(0,0,'RUNNING');	
		return;
	}
	if($self->{'current_step'} >= ($self->{'grid'}->GetNumberRows - 1)){
		$self->{'grid'}->SetCellValue($self->{'current_step'},0,$state);
		$self->{'done'} = 1;
		return;
	}
	$self->{'grid'}->SetCellValue($self->{'current_step'}++,0,$state);
	my $visible = 0;
	if (($self->{'current_step'} + 3 ) < $#{@{$self->{'plan'}}}){
		$visible = $self->{'current_step'} + 3;
	}
	else{
		$visible = $#{@{$self->{'plan'}}};
	}
			
	unless($self->{'grid'}->IsVisible($visible,0)){
		$self->{'grid'}->MakeCellVisible($visible,0);
		$self->{'grid'}->Refresh();
		$self->{'grid'}->Update();
	}
	$self->{'grid'}->SetCellValue($self->{'current_step'},0,'RUNNING');
}


sub SetProgress{
	my ($self,$value) = @_;
	return unless defined $self->{'plan'};
	$self->{'grid'}->SetCellValue($self->{'current_step'},2,$value);
	#if(defined $self->{'ctrl_'.$self->{'current_step'}.'_2'}){
	#	$self->{'ctrl_'.$self->{'current_step'}.'_2'}->SetValue($value);
	#}
}



sub Destroy{
	my ($self) = @_;
	if(defined $self->{'plan'}){
		foreach my $row (0 .. $#{@{$self->{'plan'}}}){
			if(exists $self->{'bar_'.$row}){
				$self->{'bar_'.$row}->Destroy;
				delete $self->{'bar_'.$row};
			}
		}
	}		
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	if(defined $self->{'grid'}){
		$self->{'grid'}->Destroy;
		delete $self->{'grid'};
	}
	$self->SUPER::Destroy;
}

sub DESTROY{
	my ($self) = @_;
	print ">>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;
