package SAPDB::Install::Window::InstallationView;

use SAPDB::Install::Window::PackageList;
use SAPDB::Install::Window::DBList;
use Wx::Event qw (EVT_NOTEBOOK_PAGE_CHANGED);
use Wx qw(wxHORIZONTAL wxEXPAND wxALL wxTAB_TRAVERSAL);
use SAPDB::Install::InstallRegistry;
use SAPDB::Install::Registry;
use SAPDB::Install::System;
use SAPDB::Install::Tools;


our @ISA = qw(Wx::Panel);

sub new{
	my $self = shift->SUPER::new(@_);
	$self->SetWindowStyleFlag($self->GetWindowStyleFlag | wxTAB_TRAVERSAL);
	$self->{'notebook'} = Wx::Notebook->new($self,-1);
	my $nbs = Wx::NotebookSizer->new($self->{'notebook'});
	return $self;	
}

sub init{
	my ($self) = @_;
	my ($indep_data,$indep_prog) = readIndepPath();

	eval{
		package SAPDB::Install::DummyLog;
	
		sub new{
			my $class = shift;
			my %mem;
			return bless \%mem,$class; 
	}
		sub SetMsg{
			my ($self) = @_;
			return 1;
		}
		1; 
	}; 
	
	my $log = SAPDB::Install::DummyLog->new();	
 
	my $instReg;

	eval{
		$instReg = SAPDB::Install::InstallRegistry->new("$indep_data/config/install",0,$log);
	};

	if($@){
		return undef;		
	}
	
	my @packages;
	my $kernel_versions;
	foreach my $name (sort $instReg->getPackageNames){
		my %versions = $instReg->getInstallPathes($name,1);
		$kernel_versions = \%versions if $name eq 'Database Kernel';
		foreach my $path (keys(%versions)){
			my $show_path = $path;
			$show_path = getRealPathName($path) if $^O =~ /mswin/i;
			$version = $versions{$path};
			my ($maj,$min,$cor,$bui) = ($version =~ /(\d)\.(\d)\.(\d+)\.(\d+)/);
			my $normalize_version = 0;
			if(length($cor) == 1){
				$cor = "0$cor";
				$normalize_version = 1;
			}
			if(length($bui) == 1){
				$bui = "0$bui";
				$normalize_version = 1;
			}
			if($normalize_version){
				$version = "$maj.$min.$cor.$bui";
			}
			push @packages,{'name' => $name,'version' => $version,'path' => $show_path};
		}	
	}
	$self->{'packages'} = SAPDB::Install::Window::PackageList->new($self->{'notebook'},-1);
	$self->{'packages'}->create(\@packages);
	$self->{'notebook'}->AddPage($self->{'packages'},'Software Packages',1, -1);
	my @databases;
	my %instances = getInstances();
	foreach my $instance (keys(%instances)){
		my $path =  normalizePath($instances{$instance});
		my $version = $kernel_versions->{$path} if exists $kernel_versions->{$path};
		my ($maj,$min,$cor,$bui) = ($version =~ /(\d)\.(\d)\.(\d+)\.(\d+)/);
		my $normalize_version = 0;
		if(length($cor) == 1){
			$cor = "0$cor";
			$normalize_version = 1;
		}
		if(length($bui) == 1){
			$bui = "0$bui";
			$normalize_version = 1;
		}
		if($normalize_version){
			$version = "$maj.$min.$cor.$bui";
		}
		
		$path = getRealPathName($path) if $^O =~ /mswin/i;
		push @databases,{'name' => $instance,'path' => $path,'version' => $version};
	}
	unless($#databases == -1){
		$self->{'databases'} = SAPDB::Install::Window::DBList->new($self->{'notebook'},-1);
		$self->{'databases'}->create(\@databases);
		$self->{'notebook'}->AddPage($self->{'databases'},'Database Instances',0, -1);
	}
	$self->{'notebook'}->Fit;
	my $sizer = Wx::BoxSizer->new(wxHORIZONTAL);
	$sizer->Add($self->{'notebook'},1,wxEXPAND);
	$self->SetAutoLayout(1);
	$self->SetSizerAndFit($sizer);
	EVT_NOTEBOOK_PAGE_CHANGED($self,$self->{'notebook'}->GetId,\&OnDBView);
	return 1;
}

sub OnDBView{
	my ($self,$evt) = @_;
	$evt->Skip(1);
	if ($evt->GetSelection() == 1){
		$self->{'databases'}->ShowEditors();
		$self->{'databases'}->{'grid'}->ForceRefresh();
	}
	
}



sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	while($self->{'notebook'}->RemovePage(0)){}
	if(defined $self->{'packages'}){
		$self->{'packages'}->Destroy;
		delete $self->{'packages'};
	}
	if(defined $self->{'databases'}){
		$self->{'databases'}->Destroy;
		delete $self->{'databases'};
	}
	$self->{'notebook'}->Destroy;
	delete $self->{'notebook'};
	$self->SUPER::Destroy;
}



1;