package SAPDB::Install::Window::Wizard;


use Wx::Event qw(EVT_BUTTON);
use Wx qw(wxTAB_TRAVERSAL wxEXPAND wxALIGN_CENTER_VERTICAL
	   wxALIGN_CENTER_HORIZONTAL wxDefaultSize
	   wxDefaultPosition wxHORIZONTAL wxALIGN_TOP
	   wxVERTICAL wxALIGN_BOTTOM wxCENTRE wxALIGN_RIGHT
	   wxALIGN_LEFT wxCURSOR_ARROWWAIT wxICON_ERROR
	   wxLEFT wxRIGHT wxEVT_COMMAND_BUTTON_CLICKED);

our @ISA = qw(Wx::Panel);

sub new{
	my($this ) = shift->SUPER::new(@_);
	 $this->SetWindowStyleFlag($this->GetWindowStyleFlag | wxTAB_TRAVERSAL);
	my $top_sizer = Wx::BoxSizer->new(wxHORIZONTAL);
	my $dlg_sizer = Wx::BoxSizer->new(wxVERTICAL);
	my $button_sizer = Wx::BoxSizer->new(wxHORIZONTAL);
	$this->{'btn_sizer'} = $button_sizer;
	my $id = 0;
	$this->{'page_sizer'} = Wx::BoxSizer->new(wxVERTICAL);
	$this->{'nbutton'} = Wx::Button->new($this,-1,'&Next>',wxDefaultPosition,wxDefaultSize);
	$this->{'pbutton'} = Wx::Button->new($this,-1,'<&Back',wxDefaultPosition,wxDefaultSize);
	$button_sizer->Add($this->{'pbutton'},0,wxALIGN_CENTER_LEFT | wxALIGN_CENTER_VERTICAL | wxLEFT ,30);
	$this->{'pbutton'}->Enable(0);
	$this->{'nbutton_event'} = Wx::CommandEvent->new(wxEVT_COMMAND_BUTTON_CLICKED,$this->{'nbutton'}->GetId);
	$this->{'pbutton_event'} = Wx::CommandEvent->new(wxEVT_COMMAND_BUTTON_CLICKED,$this->{'pbutton'}->GetId);
	$button_sizer->Add($this->{'nbutton'},0,wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL | wxRIGHT,30);
	$this->{'cbutton'} = Wx::Button->new($this,-1,'&Cancel',wxDefaultPosition,wxDefaultSize);
	$button_sizer->Add($this->{'cbutton'},0,wxALIGN_CENTER_RIGHT | wxALIGN_CENTER_VERTICAL | wxRIGHT,30);
	$button_sizer->SetMinSize(200,40);
	$this->{'line'} = Wx::StaticLine->new($this,-1,[0,0],[2,2]);
	$dlg_sizer->Add($this->{'page_sizer'},1,wxALIGN_TOP | wxEXPAND);
	my $line_sizer = Wx::BoxSizer->new(wxHORIZONTAL);
	$this->{'line_text'} = Wx::StaticText->new($this,-1,'MaxDB Installation');
	$this->{'line_text'}->Enable(0);
	$line_sizer->Add($this->{'line_text'},0,wxLEFT | wxRIGHT,2);
	$line_sizer->Add($this->{'line'},1,wxALIGN_CENTER_VERTICAL);	
	$dlg_sizer->Add($line_sizer,0,wxEXPAND);
	$dlg_sizer->Add($button_sizer,0, wxALIGN_CENTER_HORIZONTAL | wxALIGN_RIGHT );
	$this->{'button_line_size'} = $button_sizer->GetMinSize;
	$this->{'button_line_size'}->SetHeight($this->{'button_line_size'}->GetHeight + $this->{'line_text'}->GetSize->GetHeight);
	$top_sizer->Add($dlg_sizer,1,wxALIGN_BOTTOM | wxEXPAND);
	$this->SetSizer($top_sizer);
	$this->SetAutoLayout(1);
	$this->Layout();
	#$this->GetSizer->SetSizeHints($this);
	EVT_BUTTON($this,$this->{'pbutton'}->GetId,\&OnPrev);
	EVT_BUTTON($this,$this->{'nbutton'}->GetId,\&OnNext);
	EVT_BUTTON($this,$this->{'cbutton'}->GetId,\&OnCancel);
	$this->{'config'} = {};
	return $this;
}


sub initStep{
	my ($self) = @_;
	$self->GetParent->{'banner'}->setSubTitle($self->{'current_step'}->{'name'})
		if defined $self->GetParent->{'banner'};
	$self->setBusy(1);
	my $rc = $self->{'current_step'}->{'dlg'}->init($self->{'config'});
	return undef if($self->{'destroying'});
	$self->setBusy(0);
	if(defined $rc){
		if($rc->{'errstate'} == 1){ 
			if(defined $rc->{'errmsg'}){
				my $msgbox = Wx::MessageDialog->new($self,$rc->{'errmsg'},
					"Fatal Error", wxICON_ERROR | wxCENTRE);
					$msgbox->ShowModal();
				$msgbox->Destroy;
			}
			$self->{'error_state'} = 1;
			$self->{'config'}->{'error_state'} = 1;
			$self->gotoErrHandlerPage;
			return undef;
		}
		$self->SetConfig($rc);
		$self->{'current_step'}->{'initialized'} = 1;
		unless (exists $self->{'current_step'}->{'dlg'}->{'show_self_defined'}){
			$self->LayoutDlg;
			$self->{'current_step'}->{'dlg'}->Show(1);
		}
		$self->{'current_step'}->{'initialized'} = 1;
		if($rc->{'auto_next'}){
			$self->AddPendingEvent($self->{'nbutton_event'});
		}
		return 1;
	}
	return undef;

}

sub AddStep{
	my ($self,$step) = @_;
	
	if(defined $self->{'lastadded'}){
		$step->{'prev'} =  $self->{'lastadded'};
		$step->{'prev'}->{'next'} = $step;
	}
	else{
		$self->{'current_step'} = $step;
	}
	if($step->{'dlg'}){
		$step->{'dlg'}->Show(0);		
	}
	$self->{'lastadded'} = $step;
}

sub InsertStep{
	my ($self,$step,$name) = @_;
	
	my $relation_step = $self->{'current_step'}; 
	
	if($name =~ /\S/){
		$relation_step = $self->FindStep($name);
		unless(defined $relation_step){
			my $msgbox = Wx::MessageDialog->new($self,"Cannot find Wizard Step \"$name\"!\nInserting Step \"".$step->{'name'}."\" failed!\n",
						"Fatal Error", wxICON_ERROR | wxCENTRE);
						$msgbox->ShowModal();
			return undef;
		}
	}
	
	$step->{'prev'} = $relation_step;
	$step->{'next'} = $relation_step->{'next'};	
	$step->{'next'}->{'prev'} = $step if defined $step->{'next'};
	$relation_step->{'next'} = $step;
	return 1;
}


sub RemoveStep{
	my ($self,$name) = @_;
	
	my $relation_step = $self->{'current_step'}; 
	
	my $kill_current = 1;

		
	if($name =~ /\S/){
		unless($relation_step->{'name'} eq 'name'){
			$relation_step = $self->FindStep($name);
			unless(defined $relation_step){
				my $msgbox = Wx::MessageDialog->new($self,"Cannot find Wizard Step \"$name\"!\nRemoving Step failed!\n",
						"Fatal Error", wxICON_ERROR | wxCENTRE);
						$msgbox->ShowModal();
				return undef;
			}
			$kill_current = 0;
		}
	}

	if(defined $relation_step->{'next'}){
		if(defined $relation_step->{'prev'}){
			$relation_step->{'next'}->{'prev'} = $relation_step->{'prev'}
		}
		else{
			$relation_step->{'next'}->{'prev'} = undef;
		}
	}
	if(defined $relation_step->{'prev'}){
		if(defined $relation_step->{'next'}){
			$relation_step->{'prev'}->{'next'} = $relation_step->{'next'}
		}
		else{
			$relation_step->{'prev'}->{'next'} = undef;
		}
		if($kill_current){	
			$self->{'current_step'} = $self->{'current_step'}->{'prev'};	
		}
	}
	else{
		if($kill_current){
			$self->{'current_step'} = $self->{'current_step'}->{'next'};
		}
	}
	$relation_step->{'dlg'}->Destroy;
	delete $relation_step->{'dlg'};
}



sub FindStep{
	my ($self,$name) = @_;
	my $step = $self->{'current_step'};
	
	return $step if($step->{'name'} eq $name);
	
	#search backward
	while(1){
		$step = $step->{'prev'};
		last unless defined $step;
		if($step->{'name'} eq $name){
			return $step
		}
	}

	#search foreward
	$step = $self->{'current_step'};
	while(1){
		$step = $step->{'next'};
		last unless defined $step;
		if($step->{'name'} eq $name){
			return $step
		}
	}
	return undef;
}

sub SkipStep{
	my ($self,$name,$skip) = @_;
	my $step = $self->FindStep($name);
	return undef unless defined $step;
	if(not defined $skip || $skip){
		$step->{'skip'} = 1;
	}
	else{
		$step->{'skip'} = 0;
	}
	return 1;	
}


sub KillSteps{
	my ($self) = @_;
	my $next_step = $self->{'current_step'};
	
	# find fist step
	while(1){
		last unless defined $next_step->{'prev'};
		$next_step = $next_step->{'prev'};
	}

	# killing all steps

	while(1){
		$next_step->{'dlg'}->Destroy;	
		delete $next_step->{'dlg'};
		delete $next_step->{'name'};
		delete $next_step->{'prev'};
		last unless defined $next_step->{'next'};
		$next_step = $next_step->{'next'};
		delete $next_step->{'prev'}->{'next'};
	}
	delete $self->{'current_step'};
}


sub KillPrevSteps{
	my ($self) = @_;
	my $step = $self->{'current_step'};
	while(1){
		if (defined $step->{'prev'}){
			$step = $step->{'prev'};
			delete $step->{'next'}->{'prev'};
			$step->{'dlg'}->Destroy() if defined $step->{'dlg'};
			delete $step->{'dlg'};
		}
		else{
			last;
		}
	}
	return 1;
}


sub SetConfig{
	my ($self,$config) = @_;
	unless(ref($config) eq 'HASH'){
		return undef;
	}
	foreach my $key (keys(%$config)){
		next if($key eq 'auto_next');
		$self->{'config'}->{$key} = $config->{$key};
	}
	return 1;
}


sub GetBestSize{
	my ($self) = @_;
	my ($width,$height) = (0,0);
	if(defined $self->{'current_step'}->{'dlg'}){
		my $size = $self->{'current_step'}->{'dlg'}->GetSizer->GetMinSize;
		$width = $size->GetWidth;
		$height = $size->GetHeight;
		#print ">>>>>> page size ($width,$height)\n";
	}
	#print ">>>>> button line size = (".$self->{'button_line_size'}->GetWidth.','.$self->{'button_line_size'}->GetHeight.")\n";
	$height += $self->{'button_line_size'}->GetHeight;
	$width = $self->{'button_line_size'}->GetWidth
		if $self->{'button_line_size'}->GetWidth > $width;
	#print ">>>>> returnng wizard size ($width,$height)\n";	
	return Wx::Size->new($width,$height);		
}


sub LayoutDlg{
	my ($self) = @_;
	my $dlg = $self->{'current_step'}->{'dlg'};
	if (exists $dlg->{'show_self_defined'} && not exists $dlg->{'already_shown'}){
		if (exists $self->{'hold_dlg'} && defined $self->{'hold_dlg'}){
			$self->{'page_sizer'}->Remove($self->{'hold_dlg'});
                        $self->{'hold_dlg'}->Show(0);
			delete $self->{'hold_dlg'};
		}
		else{
			$self->{'page_sizer'}->Remove($self->{'current_step'}->{'prev'}->{'dlg'});
                	$self->{'current_step'}->{'prev'}->{'dlg'}->Show(0);
		}
	}
	my $best_size = $self->GetBestSize;
	#print ">>>>>> my \$best_size = (".$best_size->GetWidth.','.$best_size->GetHeight.")\n";
	$self->{'page_sizer'}->Remove(0);
	$dlg->Fit;
	$self->{'page_sizer'}->Add($dlg,1,wxEXPAND);
	$self->SetSizeHints($best_size->GetWidth,$best_size->GetHeight);
	if(Wx::GetTopLevelParent($self)->can('RecalcBestSize')){
		Wx::GetTopLevelParent($self)->RecalcBestSize;
	}
	$self->Layout;
	$dlg->Layout;
	if (exists $dlg->{'show_self_defined'} && not exists $dlg->{'already_shown'}){
                $dlg->Show(1);
		$dlg->{'already_shown'} = 1;
        }

}

sub init{
	my ($self) = @_;
	unless($self->{'current_step'}->{'prev'}){
		my $step = $self->{'current_step'};
		my @steps;
		while(1){
			push @steps,{'name' => $step->{'name'}};
			last unless defined $step->{'next'};
			$step = $step->{'next'};

		}
		if(defined $self->{'current_step'}->{'dlg'}){
			$self->LayoutDlg;
			$self->{'current_step'}->{'dlg'}->Show(1);
			$self->initStep;
		}
	}
	#$self->{'nbutton'}->SetDefault();
	$self->SetDefaultFocus;
	delete $self->{'lastadded'};

}

sub setBusy{
	my($self,$enable) = @_;
	if($enable){
		if(defined $self->{'current_step'}->{'dlg'} and 
				   $self->{'current_step'}->{'dlg'}->can('SetBusy')){
			
			$self->{'current_step'}->{'dlg'}->SetBusy(1);
		
		}
		$self->{'nbutton'}->Enable(0);
		$self->{'pbutton'}->Enable(0);
		if ($^O =~ /mswin/i){
			$self->{'busy_cursor'} = 
				Wx::BusyCursor->new(Wx::Cursor->new(wxCURSOR_ARROWWAIT));
		}
		else{
			$self->{'busy_cursor'} = Wx::BusyCursor->new();
		}
	}
	else{
		if(defined $self->{'current_step'}->{'dlg'} and 
				   $self->{'current_step'}->{'dlg'}->can('SetBusy')){
			
			$self->{'current_step'}->{'dlg'}->SetBusy(0);
		
		}
		if(defined $self->{'current_step'}->{'next'}){
			unless($self->{'nbutton_disabled'}){		
				$self->{'nbutton'}->Enable(1);
				unless ($^O =~ /mswin/i){
					$self->{'nbutton'}->CaptureMouse;
					$self->{'nbutton'}->ReleaseMouse;
				}
			}
		}
		if(defined $self->{'current_step'}->{'prev'}){
			$self->{'pbutton'}->Enable(1);
			unless ($^O =~ /mswin/i){
				$self->{'pbutton'}->CaptureMouse;
                                $self->{'pbutton'}->ReleaseMouse;
			}
		}
		if (exists $self->{'busy_cursor'}){
			delete $self->{'busy_cursor'};
		}
	}
	
	#
	#	refresh gui
	#
	
	if($self->{'init_ok'}){
		if(defined $SAPDB::Install::Values::RefreshGui){
			&{$SAPDB::Install::Values::RefreshGui}();
		}
	}
	else{
		$self->{'init_ok'} = 1;		
	}
	return 1;	
}




sub OnPrev{
	my ($self,$event) = @_;
	#unless(defined $self->{'current_step'}->{'prev'}){
	#	return undef;
	#}

	return undef if($self->{'destroying'});
	my $auto_back = 0;
	
	if (exists $self->{'current_step'}->{'dlg'} && 
	    exists $self->{'current_step'}->{'dlg'}->{'show_self_defined'} && 
	    exists $self->{'current_step'}->{'dlg'}->{'already_shown'}){
		delete $self->{'current_step'}->{'dlg'}->{'already_shown'};
	}
	
	if (exists $self->{'hold_dlg'} && 
	    $self->{'hold_dlg'} == $self->{'current_step'}->{'dlg'}){
		delete $self->{'hold_dlg'};	
	}

	if($self->{'current_step'}->{skip} and defined $self->{'current_step'}->{'prev'}){
		$self->{'current_step'} = $self->{'current_step'}->{'prev'};
		$self->AddPendingEvent($self->{'pbutton_event'});
		return;
	}
	
	
	if($self->{'current_step'}->{'dlg'}->can(Back)){
		my $rc = $self->{'current_step'}->{'dlg'}->Back($self->{'config'});
		return undef unless defined $rc;
		if(ref($rc) eq 'HASH' and $rc->{'auto_back'}){
			$auto_back = 1;
		}
	}

	$self->{'nbutton'}->Enable(1);
		
	if(defined $self->{'current_step'}->{'dlg'}){
		$self->{'page_sizer'}->Remove($self->{'current_step'}->{'dlg'});
		$self->{'current_step'}->{'dlg'}->Show(0);
	}
	$self->{'current_step'}->{'initialized'} = 0;
	
	if(defined $self->{'current_step'}->{'added_steps'}){
		foreach my $step_name (@{$self->{'current_step'}->{'added_steps'}}){
			$self->RemoveStep($step_name);
		}
	}
	
	$self->{'current_step'} = $self->{'current_step'}->{'prev'};

	$self->GetParent->{'banner'}->setSubTitle($self->{'current_step'}->{'name'})
		if defined $self->GetParent->{'banner'};
	
	if(defined $self->{'current_step'}->{'added_steps'}){
		foreach my $step_name (@{$self->{'current_step'}->{'added_steps'}}){
			$self->RemoveStep($step_name);
		}
		delete $self->{'current_step'}->{'added_steps'};
	}	
	
	unless(defined $self->{'current_step'}->{'prev'}){
		$self->{'pbutton'}->Enable(0);
		$self->SetDefaultFocus;
	}
	if(defined $self->{'current_step'}->{'dlg'}){
		if($self->{'current_step'}->{'dlg'}->can(BackInit)){
			my $rc = $self->{'current_step'}->{'dlg'}->BackInit($self->{'config'});
			$auto_back = 1 if($rc->{'auto_back'});
		}
		$self->LayoutDlg;
		$self->{'current_step'}->{'dlg'}->Enable(1);
		$self->{'current_step'}->{'dlg'}->Show(1);
	}
		
	if($auto_back){
		$self->AddPendingEvent($self->{'pbutton_event'});
	}
	
	return 1;
}


sub gotoErrHandlerPage{
	my ($self) = @_;
	while(1){
		if(defined $self->{'current_step'}->{'next'}){
			$self->{'page_sizer'}->Remove($self->{'current_step'}->{'dlg'});
			$self->{'current_step'}->{'dlg'}->Enable(0);
			$self->{'current_step'}->{'dlg'}->Show(0);
			$self->{'current_step'} = $self->{'current_step'}->{'next'};
		}
		else{
			$self->GetParent->{'banner'}->setSubTitle($self->{'current_step'}->{'name'})
				if defined $self->GetParent->{'banner'};
			$self->setBusy(1);
			my $rc = $self->{'current_step'}->{'dlg'}->init($self->{'config'});
			return undef if($self->{'destroying'});
			$self->setBusy(0);
			if(defined $rc){
				$self->SetConfig($rc);
				$self->{'current_step'}->{'initialized'} = 1;
				$self->LayoutDlg;
				$self->{'current_step'}->{'dlg'}->Show(1);
				if($rc->{'auto_next'}){
					$self->AddPendingEvent($self->{'nbutton_event'});
				}
			}
			last;
		}
	}
	return 1;
}


sub OnNext{
	my ($self,$event) = @_;
	#if(not defined $self->{'current_step'}->{'next'} and $self->{'current_step'}->{'initialized'}){
	#	return undef;
	#}

	return undef if($self->{'destroying'});

	$self->{'pbutton'}->Enable(1);
	
	if(	$self->{'current_step'}->{'skip'} &&
		 defined $self->{'current_step'}->{'next'}){
		
		$self->{'current_step'} = $self->{'current_step'}->{'next'};
		$self->AddPendingEvent($self->{'nbutton_event'});
		return;
	}
		
	
	unless(defined $self->{'current_step'}->{'next'}){
		$self->{'nbutton'}->Enable(0);
	}
	
	if(defined $self->{'current_step'}->{'dlg'}){
		if(0 and $self->{'error_state'} == 1 and not $self->{'current_step'}->{'initialized'}){
			if(defined $self->{'current_step'}->{'next'}){
					$self->{'page_sizer'}->Remove($self->{'current_step'}->{'dlg'});
					$self->{'current_step'}->{'dlg'}->Show(0);
					$self->{'current_step'} = $self->{'current_step'}->{'next'};
					unless(defined $self->{'current_step'}->{'next'}->{'next'}){
						$self->{'error_state'} == undef;		
					}					
					$self->AddPendingEvent($self->{'nbutton_event'});
					return;
			}
			else{
				$self->{'current_step'}->{'dlg'}->init($self->{'config'});
				return undef if($self->{'destroying'});
				#$self->AddPendingEvent($self->{'nbutton_event'});
			}
		}
		elsif($self->{'current_step'}->{'initialized'}){
			$self->setBusy(1);
			my $rc = $self->{'current_step'}->{'dlg'}->Process($self->{'config'});
			return undef if($self->{'destroying'});
			$self->setBusy(0);
			if(defined $rc){
				if($rc->{'errstate'} == 1){ 
					if(defined $rc->{'errmsg'}){
						my $msgbox;
						if(defined $rc->{'errdetail'}){
							$msgbox = Wx::MessageDialog->new($self,$rc->{'errdetail'},
								$rc->{'errmsg'}, wxICON_ERROR | wxCENTRE);
						}
						else{
							$msgbox = Wx::MessageDialog->new($self,$rc->{'errmsg'},
								"Fatal Error", wxICON_ERROR | wxCENTRE);
						}
						$msgbox->ShowModal();
						$msgbox->Destroy;
					}
					$self->{'error_state'} = 1;
					$self->{'config'}->{'error_state'} = 1;
					$self->gotoErrHandlerPage;
					return undef;
				}
				if(exists $rc->{'auto_retry'} and $rc->{'auto_retry'}){
					$self->AddPendingEvent($self->{'nbutton_event'});
					return;
				}	
				
				if(defined $rc->{'AddNewSteps'}){
					my @added_steps;
					foreach my $newstep (@{$rc->{'AddNewSteps'}}){
						$self->InsertStep($newstep->[0],$newstep->[1]);
						push @added_steps,$newstep->[0]->{'name'};
					}
					$self->{'current_step'}->{'added_steps'} = \@added_steps;
					delete $rc->{'AddNewSteps'};
				}
				unless(defined $self->{'current_step'}->{'next'}){
					$self->{'nbutton'}->Show(0) unless $self->{'log_button_enabled'};
					$self->{'cbutton'}->SetLabel('&Exit');	
					$self->{'nbutton'}->Enable(0) unless $self->{'log_button_enabled'};
					$self->{'pbutton'}->Enable(0);
					return 1;
				}
				
				$self->SetConfig($rc);
				if(defined $self->{'current_step'}->{'next'}){
					if (not exists $self->{'current_step'}->{'next'}->{'dlg'}->{'show_self_defined'}){
						if (exists $self->{'hold_dlg'} && defined  $self->{'hold_dlg'} ){
							$self->{'page_sizer'}->Remove($self->{'hold_dlg'});
                                                        $self->{'hold_dlg'}->Show(0);
							delete $self->{'hold_dlg'};	
						}
						else{
							$self->{'page_sizer'}->Remove($self->{'current_step'}->{'dlg'});
							$self->{'current_step'}->{'dlg'}->Show(0);
						}	
					}
					else{
						$self->{'current_step'}->{'dlg'}->Enable(0);
						 $self->{'hold_dlg'} = $self->{'current_step'}->{'dlg'};
					}
					
					$self->{'current_step'} = $self->{'current_step'}->{'next'};
					if($self->{'instmode'}){
						$self->{'pbutton'}->Show(0);
					}
					$self->AddPendingEvent($self->{'nbutton_event'});
					return;
				}
			}
			else{
				$self->{'current_step'}->{'dlg'}->Enable(1);
			}
		}
		else{
			$self->initStep;
		}
	}
	#$self->{'nbutton'}->Enable(1);
	$self->SetDefaultFocus;
}

sub SetDefaultFocus{
	my ($self) = @_;
	if($self->{'nbutton'}->IsEnabled){
		$self->{'nbutton'}->SetFocus;
	}
	elsif($self->{'pbutton'}->IsEnabled and $self->{'pbutton'}->IsShown){
		$self->{'pbutton'}->SetFocus;
	}
	else{
		$self->{'cbutton'}->SetFocus;
	}
}
	

sub OnCancel{
	my ($self,$event) = @_;
	my $label = $self->{'cbutton'}->GetLabel;
	$label =~ s/&//g;

	if(Wx::GetTopLevelParent($self)->can('EndModal')){
		my $dlg = Wx::GetTopLevelParent($self);
		
		if($label eq 'Exit'){
			$dlg->EndModal(1);		
		}
		else{
			
			$dlg->Close();
		}
	}
	elsif($label eq 'Exit'){
		Wx::GetTopLevelParent($self)->Close(1);
	}
	else{
		Wx::GetTopLevelParent($self)->Close();
	}
}

sub SetInstallMode{
	my ($self,$enable,$label) = @_;
	if($enable != 0){
		return if $self->{'instmode'}; 
		#$self->{'pbutton'}->Show(0);
		$self->{'nbutton'}->SetLabel(defined $label ? $label : "&Install");
		$self->{'instmode'} = 1;
	}
	else{
		return unless $self->{'instmode'}; 
		$self->{'pbutton'}->Show(1);
		$self->{'nbutton'}->SetLabel("&Next>");
		delete $self->{'instmode'};
	}
	my $nbutton_minsize = $self->{'nbutton'}->GetBestSize();
	$self->{'btn_sizer'}->SetItemMinSize(1,$nbutton_minsize->GetWidth, $nbutton_minsize->GetHeight);
}

sub SetRetryMode{
	my ($self,$enable) = @_;
	if($enable != 0){
		return if $self->{'retrymode'}; 
		$self->{'nbutton'}->SetLabel("&Retry");
		$self->{'nbutton'}->Enable(1);
		unless ($^O =~ /mswin/i){
			$self->{'nbutton'}->CaptureMouse;
			$self->{'nbutton'}->ReleaseMouse;
		}
		$self->{'retrymode'} = 1;
	}
	else{
		return unless $self->{'retrymode'}; 
		$self->{'nbutton'}->SetLabel("&Install");
		$self->{'nbutton'}->Enable(0);
		delete $self->{'retrymode'};
	}
	my $nbutton_minsize = $self->{'nbutton'}->GetBestSize();
	$self->{'btn_sizer'}->SetItemMinSize(1,$nbutton_minsize->GetWidth, $nbutton_minsize->GetHeight);
}


sub LogButton{
	my ($self,$label,$evt_handler) = @_;
	$self->{'nbutton'}->SetLabel($label);
	
	EVT_BUTTON($self,$self->{'nbutton'}->GetId,$evt_handler);
	if($self->{'pbutton'}->IsShown){
		$self->{'pbutton'}->Show(0);
	}
	
	unless ($self->{'nbutton'}->IsShown){
		$self->{'nbutton'}->Show(1);
	}	
	
	unless ($self->{'nbutton'}->IsEnabled){	
		$self->{'nbutton'}->Enable(1);
		unless ($^O =~ /mswin/i){
			$self->CaptureMouse;
		    $self->ReleaseMouse;
		}
	}

	my $nbutton_minsize = $self->{'nbutton'}->GetBestSize();
	$self->{'btn_sizer'}->SetItemMinSize(1,$nbutton_minsize->GetWidth, $nbutton_minsize->GetHeight);
	$self->Layout;
	$self->{'log_button_enabled'} = 1;
}


sub Destroy{
	my ($self) = @_;
	$self->{'destroying'} = 1;
	delete $self->{'btn_sizer'};
	while($self->{'page_sizer'}->Remove(0)){}
	$self->KillSteps;
	while($self->GetSizer->Remove(0)){}
	$self->{'nbutton'}->Destroy;
	delete $self->{'nbutton'};
	$self->{'pbutton'}->Destroy;
	delete $self->{'pbutton'};
	$self->{'cbutton'}->Destroy;
	delete $self->{'cbutton'};
	$self->{'line_text'}->Destroy;
	delete $self->{'line_text'};
	$self->{'line'}->Destroy;
	delete $self->{'line'};
	$self->SUPER::Destroy;
}


sub DESTROY{
	my ($self) = @_;
	print ">>>> DESTROY $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;
