#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_27/sys/src/install/perl/SAPDB/Install/Wizard/Dialog/Components.pm#1 $
# $DateTime: 2006/04/27 14:44:55 $
# $Change: 124736 $
#
# Desc: start SAP DB installation/update/patch/instanceupgrade,
# called by SDBINST/SDBUPD
#

package SAPDB::Install::Wizard::Dialog::Components;

use Wx::Event qw (EVT_ENTER_WINDOW EVT_BUTTON
				  EVT_RADIOBUTTON);
use Wx qw (
	wxICON_ERROR wxCENTRE wxVERTICAL wxDefaultSize 
	wxBOTTOM wxHORIZONTAL wxEXPAND
	wxALL wxLEFT wxTOP wxALIGN_CENTER_VERTICAL
	wxALIGN_RIGHT wxRIGHT wxSYS_SYSTEM_FONT);

use SAPDB::Install::Dialog::PackageView;
use SAPDB::Install::Control::StaticText;
use SAPDB::Install::Wizard::Dialog::Base;
use SAPDB::Install::Wizard::Dialog::DBHandling;
use SAPDB::Install::Wizard::Dialog::AdvancedComponents;
use SAPDB::Install::PackageCollector;
use SAPDB::Install::ObjMngr;
use SAPDB::Install::Log;
use SAPDB::Install::Registry;
use SAPDB::Install::InstallRegistry;
use SAPDB::Install::Migrate::DBRegistrations;
use SAPDB::Install::Migrate::Registry;
	
our @ISA = qw (SAPDB::Install::Wizard::Dialog::Base);

our $components = {
	'server' =>	{
				'desc' =>'Installs all server and client software components. '.
						 'Includes all basic command line tools for executing '.
						 'SQL statements and administering databases. '.
						 'Minimum attention required. Essential if you want '.
						 'to create a new or upgrade an existing database.',
 				'label' => '&Server + Client',
				'seq' => 1,
				'leaf_packages' => [
							'Database Kernel','Loader','DB Analyzer','Webtools',
						    	'JDBC','CPC SDK *','PCR *','SQLDBC *','SAP Utilities',
							'SQL Studio','Database Manager','Messages','WebSQL',
							'WebDAV Servlet', 'Synchronization Manager',
							'Database Connectivity',
							'ODBC','APO COM']
			},
	'client' => {
				'desc' =>	'Installs the client software components only. '.
							'Suitable for updating the client software of '.
							'an existing software installation.',
				'label' => 'C&lient',
				'seq' => 2,
				'leaf_packages' => [	
							'Loader','DB Analyzer','Webtools','SAP Utilities',
						    	'JDBC','CPC SDK *','PCR *','SQLDBC *',
						   	'SQL Studio','Database Manager','Messages',
						  	'Database Connectivity','WebSQL','WebDAV Servlet',
							'Synchronization Manager','ODBC'] 

			},
	'custom' => {
				'desc' =>	'Installs a definable subset of software components. '.
							'Suitable for experienced users only. '.
							'For detailed information see the Installation Manual.',
				'label' => 'C&ustom',
				'seq' => 2,
				'leaf_packages' => [
							'Database Kernel','Loader','DB Analyzer','Webtools',
							'JDBC','CPC SDK *','PCR *','SQLDBC *','ODBC',
							'SQL Studio','Database Manager','Messages',
							'WebSQL','WebDAV Servlet', 'Synchronization Manager',
							'Database Connectivity',
							'APO COM','SAP Utilities'] 
	},


	#'conn' => {
	#			'desc' => 	"Install the Connectivity Components ".
	#						"if you want to use external programs connecting ".
	#						"MaxDB/SAP DB via ODBC, JDBC ... ".
	#						"This Components contain also the available ".
	#						"MaxDB SDKs. Thus if You want to develop Applications ".
	#						"working with MaxDB, You should select this.",
	#						
	#			'label' => 'C&onnectivity',
	#			'seq' => 3,
	#			'leaf_packages' => ['ODBC','JDBC','CPC SDK *','PCR *','Database Connectivity'] 
	#	}
};


my $default_component = 'server';


sub new{
	my($this ) = shift->SUPER::new(@_);
	$this->{'static_box_components'} = Wx::StaticBox->new($this,-1,'Component Groups');
	$this->{'static_box_desc'} = Wx::StaticBox->new($this,-1,'Description');
	my $sizer = Wx::BoxSizer->new(wxHORIZONTAL);
	my $rb_sizer = Wx::StaticBoxSizer->new($this->{'static_box_components'},wxVERTICAL);
	my $text_sizer = Wx::StaticBoxSizer->new($this->{'static_box_desc'},wxVERTICAL); 
	my $left_sizer = Wx::BoxSizer->new(wxVERTICAL);
	
	my $abutton = Wx::Button->new($this,-1,'&Details...',[0,0],[-1,-1]);
	$this->{'button'} = $abutton;
	EVT_BUTTON($this,$abutton->GetId,\&OnDetails);
 	
 	my $index = 0;

	my @components = sort {$components->{$a}->{'seq'} <=> $components->{$b}->{'seq'} } keys(%$components);
	if($^O !~ /mswin/i){
		$this->{'text_desc'} = SAPDB::Install::Control::StaticText->new($this,-1,[0,0],[250,250]);
		$this->{'text_desc'}->{'auto_resize'} = 0;
		$this->{'text_desc'}->{'auto_split'} = 1;
	}
	else{
		$this->{'text_desc'} = Wx::StaticText->new($this,-1,'',[0,0],[200,250]);
	}
	$text_sizer->Add($this->{'text_desc'},1,wxEXPAND | wxALL,10 );
	
	foreach my $comp (@components){
		$this->{'rb_'.$comp} = Wx::RadioButton->new($this,
							-1,$components->{$comp}->{'label'},[0,0],[-1,-1]);
		
		if($default_component eq $comp){
			$this->{'rb_'.$comp}->SetValue(1);
			my $label = $components->{$comp}->{'label'};
			$label =~ s/&//g;
			#$this->{'static_box_desc'}->SetLabel('Group description: '.$label);
			$this->{'text_desc'}->SetLabel($components->{$comp}->{'desc'});
		}	
		
		if($index == 0){
			$rb_sizer->Add($this->{'rb_'.$comp},1,wxEXPAND | wxLEFT | wxTOP, 30);
		}
		elsif($index == $#components){
			$rb_sizer->Add($this->{'rb_'.$comp},1,wxEXPAND | wxLEFT | wxBOTTOM, 30);
		}
		else{
			$rb_sizer->Add($this->{'rb_'.$comp},1,wxEXPAND | wxLEFT , 30);
		}
		$rb_sizer->CalcMin;
		EVT_RADIOBUTTON($this,$this->{'rb_'.$comp}->GetId,\&OnRBSelect);
		#EVT_ENTER_WINDOW($this->{'rb_'.$comp},\&OnEnter);
		$index++;
	}
	$left_sizer->Add($rb_sizer,1,wxEXPAND); 
	$left_sizer->Add($abutton,0,  wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxTOP,20);	
	$sizer->Add($left_sizer,1,wxEXPAND | wxALL , 40);
	$sizer->Add($text_sizer,1,wxEXPAND | wxALL ,40);
	#$sizer->CalcMin;
	#$rb_sizer->Add($abutton,0,  wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxRIGHT | wxBOTTOM, 10);
	$this->SetSizerAndFit($sizer);
	$this->SetAutoLayout(1);
	$this->Layout();
	return $this;
}

sub OnDetails{
	my ($self,$evt) = @_;
	my @components;
	foreach my $comp (keys(%$components)){
		if($self->{'rb_'.$comp}->GetValue){
			push  @components,$comp;
		}	
	}
	
	#
	#	reset selections
	#
	
	foreach my $pname (keys(%{$self->{'config'}->{'collector'}->package_index})){
		$self->{'config'}->{'collector'}->package_index->{$pname}->{'tree_ref'}->{'selected'} = 0;  
	}

	foreach my $comp (@components){
		foreach my $leaf (@{$components->{$comp}->{'leaf_packages'}}){
			if ($leaf =~ /\*/){
				foreach my $pname (keys(%{$self->{'config'}->{'collector'}->package_index})){
					my $pattern = $leaf;
					$pattern =~ s/\*/\.\+/g;
					$pattern = '^'.$pattern.'$';
					if($pname =~ /$pattern/){
						next unless exists $self->{'config'}->{'collector'}->package_index->{$pname};
						$self->{'config'}->{'collector'}->package_index->{$pname}->{'tree_ref'}->{'selected'} = 1;  
					}
				}
			}
			else{
				next unless exists $self->{'config'}->{'collector'}->package_index->{$leaf};
				$self->{'config'}->{'collector'}->package_index->{$leaf}->{'tree_ref'}->{'selected'} = 1;  
			}
		}
	}

	my @packages;
	foreach my $package (sort @{$self->{'config'}->{'collector'}->GetPackageList(undef,1)}){
		my $packagedata = $self->{'config'}->{'collector'}->package_index->{$package};
		my $name = $packagedata->{'DISPLAY_NAME'};
		my $size = $packagedata->{'SIZE'};
		if(defined $size){
			my $size_kb = $size / 1024;
			my $size_mb = int($size_kb / 1024);
			my $size_mb_rest = int(($size_kb % 1024) * 1000 / 1024);
			
			$size_mb_rest = '0' x  (3 - length($size_mb_rest)).$size_mb_rest;
			$size = $size_mb.'.'.$size_mb_rest.'  ';
		}
		else{
			$size = '<unknown>  ';
		}
		$name = $packagedata->{'PACKAGE_NAME'} unless $name =~ /\S/;
		push @packages,{'name' => $name,'version' => $packagedata->{'SOFTWARE_VERSION'},'size' => $size};
	}

	SAPDB::Install::Dialog::PackageView->new(\@packages,$self,-1,[0,0],[-1,-1])->Destroy;
	$self->{'button'}->SetFocus;
	#$self->SetMsg("Missing Feature",'Sorry, it\'s not yet implemented!');
	
	return 1;
}

#sub OnAdvanced{
#	my ($self,$evt) = @_;
#	$self->{'advanced_mode'} = 1;
#	$self->GetWizard->AddPendingEvent($self->GetWizard->{'nbutton_event'});
#	return 1;
#}

sub init(){
	my ($self,$config) = @_;
	$self->Layout;
	$self->Show(1);
	#
	#	refresh gui
	#
	if(defined $SAPDB::Install::Values::RefreshGui){
		&{$SAPDB::Install::Values::RefreshGui}();
	}
	$self->{'config'} = $config;
	unless (defined $config->{'container'}){
		$config->{'container'} = SAPDB::Install::ObjMngr->new();
	}
	unless(defined $config->{'container'}->Log){
		$config->{'container'}->Log(SAPDB::Install::Log->new());
	}
	unless(defined $config->{'collector'}){
		$config->{'collector'} = SAPDB::Install::PackageCollector->new();
		$config->{'collector'}->log($config->{'container'}->Log);
		$config->{'collector'}->archive_dir($SAPDB::Install::Config{'RuntimeDir'});
	}
	
	unless(defined $config->{'container'}->InstReg){
				
		#
		#	set these values for compatibility reasons
		#	with older packages (especially PCR)
		#

		$SAPDB::Install::Values::log = $config->{'container'}->Log;
		$config->{'container'}->SetRefLog(\$SAPDB::Install::Values::log);
		$SAPDB::Install::StartInstall::log = $config->{'container'}->Log;		
		$config->{'container'}->SetRefLog(\$SAPDB::Install::StartInstall::log);
		
		my ($indep_data,$indep_prog) = readIndepPath();
		if($indep_data =~ /\S/ and -d "$indep_data/config/install"){
			eval{
				$config->{'container'}->Log->LogPath("$indep_data/wrk");
				$config->{'container'}->InstReg(SAPDB::Install::InstallRegistry->new("$indep_data/config/install",1,$config->{'container'}->Log)) unless defined $config->{'container'}->InstReg;
			};
			if($@){
				$self->SetError("Error loading InstallRegistry",$@);
				return {'errstate' => 1};
			}
			
			#
			#	same reasons as log object above
			#
			$SAPDB::Install::StartInstall::instRegistry = $config->{'container'}->InstReg;
			$config->{'container'}->SetRefInstReg(\$SAPDB::Install::StartInstall::instRegistry);			
		}
	}
	
	unless($config->{'collector'}->GenPackageTree($config->{'container'}->InstReg)){
			$self->SetError("Finding Packages failed!",$config->{'collector'}->last_error);
			return {'errstate' => 1};
	}

	unless(exists $config->{'collector'}->package_index->{'Database Kernel'}){
		$self->{'rb_server'}->SetValue(0);
		$self->{'server_disabled'} = 1;
		$self->{'rb_client'}->SetValue(1);
		$self->{'text_desc'}->SetLabel($components->{'client'}->{'desc'});
		$self->{'rb_server'}->Enable(0);
	}	
		
	$self->{'initialized'} = 1;
	$self->Layout;
	return {}; 
}

sub Process{
	my ($self,$config) = @_;
	my %rv;
	my @components;
	my $server_selected = 0;
	foreach my $comp (keys(%$components)){
		if($self->{'rb_'.$comp}->GetValue){
			$self->{'advanced_mode'} = 1 if ($comp eq 'custom');
			$server_selected = 1 if($comp eq 'server');
			push  @components,$comp;
		}	
	}
	$rv{'components'} =  \@components;
	$rv{'instmode'} = 'Custom';
	
	
	#
	#	reset selections
	#
	
	foreach my $pname (keys(%{$config->{'collector'}->package_index})){
		$config->{'collector'}->package_index->{$pname}->{'tree_ref'}->{'selected'} = 0;  
	}


	#
	#	set component selections
	#

	
	foreach my $comp (@components){
		foreach my $leaf (@{$components->{$comp}->{'leaf_packages'}}){
			if ($leaf =~ /\*/){
				foreach my $pname (keys(%{$config->{'collector'}->package_index})){
					my $pattern = $leaf;
					$pattern =~ s/\*/\.\+/g;
					$pattern = '^'.$pattern.'$';
					if($pname =~ /$pattern/){
						next unless exists $config->{'collector'}->package_index->{$pname};
						$config->{'collector'}->package_index->{$pname}->{'tree_ref'}->{'selected'} = 1;  
					}
				}
			}
			else{
				next unless exists $config->{'collector'}->package_index->{$leaf};
				$config->{'collector'}->package_index->{$leaf}->{'tree_ref'}->{'selected'} = 1;  
			}
		}
	}

	
	if($self->{'advanced_mode'}){
			$rv{'AddNewSteps'} = [
							[{
								'name' => 'Select/deselect inidividual components',
								'dlg' => SAPDB::Install::Wizard::Dialog::AdvancedComponents->new($self->GetWizard,
									-1,[0,0],wxDefaultSize)
							},undef]
				];
		delete $self->{'advanced_mode'};
		return \%rv;
	}
	
	if($#components == -1){
		$self->SetError("No Component selected",'Please Select at least one Component!');
		return undef;
	}
	
	if($server_selected){ 	
			$rv{'AddNewSteps'} = [
									[{
										'name' => 'Choose installation type',
										'dlg' => SAPDB::Install::Wizard::Dialog::DBHandling->new($self->GetWizard,
											-1,[0,0],wxDefaultSize)
									},undef]
								];
	}

	$rv{'packages'} = $config->{'collector'}->GenPackages;
	if(defined $config->{'container'}->InstReg){
		migrateReg($config->{'container'}->InstReg,@{$rv{'packages'}});
	}
	return \%rv;
}

sub OnEnter{
	my ($cb,$event) = @_;
	my $label = $cb->GetLabel;
	$label =~ s/&//g;
	my $desc;
	foreach my $comp (keys(%$components)){
		my $clabel = $components->{$comp}->{'label'};
		$clabel =~ s/&//g;
		if($clabel eq $label){
			$desc = $components->{$comp}->{'desc'};
			last;
		}
	}
	$label =~ s/&//g;
	#$cb->GetParent->{'static_box_desc'}->SetLabel('Group description: '.$label);	
	$cb->GetParent->{'text_desc'}->SetLabel($desc);
	$cb->GetParent->Layout();
	$event->Skip(1);
}	

sub OnRBSelect{
	my ($self,$event) = @_;
	my $rb = $self->FindWindow($event->GetId);
	my $label = $rb->GetLabel;
	$label =~ s/&//g;
	my $desc;
	foreach my $comp (keys(%$components)){
		my $clabel = $components->{$comp}->{'label'};
		$clabel =~ s/&//g;
		if($clabel eq $label){
			$desc = $components->{$comp}->{'desc'};
			last;
		}
	}
	$label =~ s/&//g;
	if($label =~ /custom/i){
		$self->{'button'}->Enable(0);
	}
	else{
		$self->{'button'}->Enable(1);
	}
	#$self->{'static_box_desc'}->SetLabel('Group description: '.$label);	
	
	$self->{'text_desc'}->SetLabel($desc);
	$self->Layout;
	
	$event->Skip(1);
}

sub SetBusy{
	my ($self,$busy) = @_;
	if(not defined $busy or $busy){
		$self->Enable(0);
	}
	else{
		$self->Enable(1);		
		$self->{'rb_server'}->Enable(0)
			if $self->{'server_disabled'};
	}
	return 1;
}


sub BackInit{
	my ($self,$config) = @_;
	if(defined $config->{'packages'}){
		foreach my $package (@{$config->{'packages'}}){
			$SAPDB::Install::PackageObj::SELFOBJ[$package->ID] = undef;
		}
		delete $config->{'packages'};
	}
	return 1;
}
	

sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	if(defined $self->{'static_box_desc'}){
		$self->{'static_box_desc'}->Destroy;
		delete $self->{'static_box_desc'};
	}
	if(defined $self->{'static_box_components'}){
		$self->{'static_box_components'}->Destroy;
		delete $self->{'static_box_components'};
	}
	foreach my $comp (@components){
		if(defined $self->{'rb_'.$comp}){
			$self->{'rb_'.$comp}->Destroy;
			delete $self->{'rb_'.$comp};
		}
	}
	if(defined $self->{'button'}){
		$self->{'button'}->Destroy;
		delete $self->{'button'};
	}
	$self->SUPER::Destroy;
}

sub DESTROY{
	my ($self) = @_;
	print ">>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;
