#SAY --- Installing SYSINFO catalog views ...
#
#
#    ========== licence begin  GPL
#    Copyright (c) 2000-2005 SAP AG
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#    ========== licence end

from installib import *

def createSysinfoSynonym (session, view):
    stopIfSqlNotOK (session, 'grant select on sysinfo.' + view
        + ' to public')
    stopIfSqlNotErr (session, -4007, 'drop public synonym ' + view)
    stopIfSqlNotOK (session, 'create public synonym ' + view
        + ' for sysinfo.' + view + ' internal')

def install (session,options):
    """Installing SYSINFO tables
    """
    switchToDefaultSchema (session, options)

    stopIfSqlNotErr (session, -4003, 'DROP USER SYSINFO')

    stopIfSqlNotErr (session, -6006, 'CREATE SCHEMA SYSINFO')

    sqlOK (session, "COMMIT")

    switchToSchema (session, 'SYSINFO')

    loaderCmd (session, 'INSTALLATION ON')

    loaderCmd (session, 'AUTOCOMMIT OFF')

    createReplaceView (session, 'SYSINFO.ACTIVECONFIGURATION',
        """AS SELECT *
           FROM SYSDD.ACTIVECONFIGURATION
        INTERNAL""")

    createSysinfoSynonym (session, 'ACTIVECONFIGURATION')


    createReplaceView (session, 'SYSINFO.BACKUPTHREADS',
        """AS SELECT *
           FROM SYSDD.BACKUPTHREADS
        INTERNAL""")

    createSysinfoSynonym (session, 'BACKUPTHREADS')


    createReplaceView (session, 'SYSINFO.CACHESTATISTICS',
        """AS SELECT *
           FROM SYSDD.CACHESTATISTICS
        INTERNAL""")

    createSysinfoSynonym (session, 'CACHESTATISTICS')


    createReplaceView (session, 'SYSINFO.CATALOGCACHESTATISTICS',
        """AS SELECT *
           FROM SYSDD.CATALOGCACHESTATISTICS
        INTERNAL""")

    createSysinfoSynonym (session, 'CATALOGCACHESTATISTICS')


    createReplaceView (session, 'SYSINFO.COMMANDCACHESTATISTICS',
        """AS SELECT *
           FROM SYSDD.COMMANDCACHESTATISTICS
           WHERE USER IN(SELECT NAME
                         FROM SYSCATALOG.AUTHORIZATION
                         WHERE NAME = USERGROUP
                         AND USERKIND IN(x'01', x'04', x'05'))
        INTERNAL""")

    createSysinfoSynonym (session, 'COMMANDCACHESTATISTICS')


    createReplaceView (session, 'SYSINFO.COMMANDCACHESTATISTICSRESET',
        """AS SELECT *
           FROM SYSDD.COMMANDCACHESTATISTICSRESET
           WHERE USER IN(SELECT NAME
                         FROM SYSCATALOG.AUTHORIZATION
                         WHERE NAME = USERGROUP
                         AND USERKIND IN(x'01', x'04', x'05'))
        INTERNAL""")

    createSysinfoSynonym (session, 'COMMANDCACHESTATISTICSRESET')


    createReplaceView (session, 'SYSINFO.COMMANDSTATISTICS',
        """AS SELECT *
           FROM SYSDD.COMMANDSTATISTICS
           WHERE USER IN(SELECT NAME
                         FROM SYSCATALOG.AUTHORIZATION
                         WHERE NAME = USERGROUP
                         AND USERKIND IN(x'01', x'04', x'05'))
        INTERNAL""")

    createSysinfoSynonym (session, 'COMMANDSTATISTICS')


    createReplaceView (session, 'SYSINFO.COMMANDSTATISTICSRESET',
        """AS SELECT *
           FROM SYSDD.COMMANDSTATISTICSRESET
           WHERE USER IN(SELECT NAME
                         FROM SYSCATALOG.AUTHORIZATION
                         WHERE NAME = USERGROUP
                         AND USERKIND IN(x'01', x'04', x'05'))
        INTERNAL""")

    createSysinfoSynonym (session, 'COMMANDSTATISTICSRESET')


    createReplaceView (session, 'SYSINFO.CONSISTENTVIEWS',
        """(CONSISTENTVIEW, CREATORTRANSCOUNT, OPENTRANS, OPENDATE, LASTUSED, CANCELED)
        AS SELECT
              CONSISTENT_VIEW, CREATOR_TRANSCOUNT, OPEN_TRANS, TIMESTAMP ("DATE", "TIME"), LAST_USED, CANCELLED
           FROM SYSDD.CONSISTENTVIEWS
        INTERNAL""")

    createSysinfoSynonym (session, 'CONSISTENTVIEWS')


    createReplaceView (session, 'SYSINFO.DATACACHE',
        """AS SELECT *
           FROM SYSDD.DATACACHE
        INTERNAL""")

    createSysinfoSynonym (session, 'DATACACHE')


    createReplaceView (session, 'SYSINFO.DATASTATISTICS',
        """AS SELECT *
           FROM SYSDD.DATASTATISTICS
        INTERNAL""")

    createSysinfoSynonym (session, 'DATASTATISTICS')


    createReplaceView (session, 'SYSINFO.DATASTATISTICSRESET',
        """AS SELECT *
           FROM SYSDD.DATASTATISTICSRESET
        INTERNAL""")

    createSysinfoSynonym (session, 'DATASTATISTICSRESET')


    createReplaceView (session, 'SYSINFO.DATAVOLUMES',
        """AS SELECT *
           FROM SYSDD.DATAVOLUMES
        INTERNAL""")

    createSysinfoSynonym (session, 'DATAVOLUMES')


    createReplaceView (session, 'SYSINFO.FILEDIRECTORIES',
        """AS SELECT *
           FROM SYSDD.FILEDIRECTORIES
        INTERNAL""")

    createSysinfoSynonym (session, 'FILEDIRECTORIES')


    createReplaceView (session, 'SYSINFO.FILES',
        """AS SELECT *
           FROM SYSDD.FILES
        INTERNAL""")

    createSysinfoSynonym (session, 'FILES')


    createReplaceView (session, 'SYSINFO.HOTSTANDBYCOMPONENT',
        """AS SELECT *
           FROM SYSDD.HOTSTANDBYCOMPONENT
        INTERNAL""")

    createSysinfoSynonym (session, 'HOTSTANDBYCOMPONENT')


    createReplaceView (session, 'SYSINFO.HOTSTANDBYGROUP',
        """AS SELECT *
           FROM SYSDD.HOTSTANDBYGROUP
        INTERNAL""")

    createSysinfoSynonym (session, 'HOTSTANDBYGROUP')


    createReplaceView (session, 'SYSINFO.INSTANCE ',
        """AS SELECT *
           FROM SYSDD.INSTANCE
        INTERNAL""")

    createSysinfoSynonym (session, 'INSTANCE')


    createReplaceView (session, 'SYSINFO.IOBUFFERCACHES',
        """AS SELECT *
           FROM SYSDD.IOBUFFERCACHES
        INTERNAL""")

    createSysinfoSynonym (session, 'IOBUFFERCACHES')


    createReplaceView (session, 'SYSINFO.IOTHREADSTATISTICS',
        """AS SELECT *
           FROM SYSDD.IOTHREADSTATISTICS
        INTERNAL""")

    createSysinfoSynonym (session, 'IOTHREADSTATISTICS')


    createReplaceView (session, 'SYSINFO.IOTHREADSTATISTICSRESET',
        """AS SELECT *
           FROM SYSDD.IOTHREADSTATISTICSRESET
        INTERNAL""")

    createSysinfoSynonym (session, 'IOTHREADSTATISTICSRESET')


    createReplaceView (session, 'SYSINFO.LOGVOLUMES',
        """AS SELECT *
           FROM SYSDD.LOGVOLUMES
        INTERNAL""")

    createSysinfoSynonym (session, 'LOGVOLUMES')


    createReplaceView (session, 'SYSINFO.MACHINECONFIGURATION',
        """AS SELECT *
           FROM SYSDD.MACHINECONFIGURATION
        INTERNAL""")

    createSysinfoSynonym (session, 'MACHINECONFIGURATION')


    createReplaceView (session, 'SYSINFO.MACHINEUTILIZATION',
        """AS SELECT *
           FROM SYSDD.MACHINEUTILIZATION
        INTERNAL""")

    createSysinfoSynonym (session, 'MACHINEUTILIZATION')


    createReplaceView (session, 'SYSINFO.MEMORYALLOCATORSTATISTICS',
        """AS SELECT *
           FROM SYSDD.MEMORYALLOCATORSTATISTICS
        INTERNAL""")

    createSysinfoSynonym (session, 'MEMORYALLOCATORSTATISTICS')


    createReplaceView (session, 'SYSINFO.MEMORYHOLDERS',
        """AS SELECT *
           FROM SYSDD.MEMORYHOLDERS
        INTERNAL""")

    createSysinfoSynonym (session, 'MEMORYHOLDERS')

    createReplaceView (session, 'SYSINFO.OBJECTLOCKS',
        """AS SELECT *
           FROM SYSDD.OBJECTLOCKS
        INTERNAL""")

    createSysinfoSynonym (session, 'OBJECTLOCKS')

    createReplaceView (session, 'SYSINFO.OMSDIAGNOSE',
        """AS SELECT *
           FROM SYSDD.OMSDIAGNOSE
        INTERNAL""")

    createSysinfoSynonym (session, 'OMSDIAGNOSE')

    createReplaceView (session, 'SYSINFO.OMSLOCKS',
        """AS SELECT *
           FROM SYSDD.OMSLOCKS
        INTERNAL""")

    createSysinfoSynonym (session, 'OMSLOCKS')


    createReplaceView (session, 'SYSINFO.OPTIMIZERINFORMATION',
        """AS SELECT *
           FROM SYSDD.OPTIMIZERINFORMATION
        INTERNAL""")

    createSysinfoSynonym (session, 'OPTIMIZERINFORMATION')

    createReplaceView (session, 'SYSINFO.READERWRITERLOCKINFORMATION',
        """AS SELECT *
           FROM SYSDD.READERWRITERLOCKINFORMATION
        INTERNAL""")

    createSysinfoSynonym (session, 'READERWRITERLOCKINFORMATION')

    createReplaceView (session, 'SYSINFO.READERWRITERLOCKSTATISTICS',
        """AS SELECT *
           FROM SYSDD.READERWRITERLOCKSTATISTICS
        INTERNAL""")

    createSysinfoSynonym (session, 'READERWRITERLOCKSTATISTICS')

    createReplaceView (session, 'SYSINFO.READERWRITERLOCKSTATISTICSRESET',
        """AS SELECT *
           FROM SYSDD.READERWRITERLOCKSTATISTICSRESET
        INTERNAL""")

    createSysinfoSynonym (session, 'READERWRITERLOCKSTATISTICSRESET')

    createReplaceView (session, 'SYSINFO.READERWRITERLOCKWAITINGTASKS',
        """AS SELECT *
           FROM SYSDD.READERWRITERLOCKWAITINGTASKS
        INTERNAL""")

    createSysinfoSynonym (session, 'READERWRITERLOCKWAITINGTASKS')

    createReplaceView (session, 'SYSINFO.REGIONINFORMATION',
        """AS SELECT *
           FROM SYSDD.REGIONINFORMATION
        INTERNAL""")

    createSysinfoSynonym (session, 'REGIONINFORMATION')

    createReplaceView (session, 'SYSINFO.REGIONSTATISTICS',
        """AS SELECT *
           FROM SYSDD.REGIONSTATISTICS
        INTERNAL""")

    createSysinfoSynonym (session, 'REGIONSTATISTICS')

    createReplaceView (session, 'SYSINFO.REGIONSTATISTICSRESET',
        """AS SELECT *
           FROM SYSDD.REGIONSTATISTICSRESET
        INTERNAL""")

    createSysinfoSynonym (session, 'REGIONSTATISTICSRESET')

    createReplaceView (session, 'SYSINFO.RESTARTINFORMATION',
        """AS SELECT *
           FROM SYSDD.RESTARTINFORMATION
        INTERNAL""")

    createSysinfoSynonym (session, 'RESTARTINFORMATION')

    createReplaceView (session, 'SYSINFO.SESSIONS',
        """AS SELECT *
           FROM SYSDD.SESSIONS
        INTERNAL""")

    createSysinfoSynonym (session, 'SESSIONS')

    createReplaceView (session, 'SYSINFO.SNAPSHOTS',
        """AS SELECT *
           FROM SYSDD.SNAPSHOTS
        INTERNAL""")

    createSysinfoSynonym (session, 'SNAPSHOTS')

    createReplaceView (session, 'SYSINFO.SPINLOCKPOOLSTATISTICS',
        """AS SELECT *
           FROM SYSDD.SPINLOCKPOOLSTATISTICS
        INTERNAL""")

    createSysinfoSynonym (session, 'SPINLOCKPOOLSTATISTICS')

    createReplaceView (session, 'SYSINFO.SPINLOCKPOOLSTATISTICSRESET',
        """AS SELECT *
           FROM SYSDD.SPINLOCKPOOLSTATISTICSRESET
        INTERNAL""")

    createSysinfoSynonym (session, 'SPINLOCKPOOLSTATISTICSRESET')

    createReplaceView (session, 'SYSINFO.SPINLOCKSTATISTICS',
        """AS SELECT *
           FROM SYSDD.SPINLOCKSTATISTICS
        INTERNAL""")

    createSysinfoSynonym (session, 'SPINLOCKSTATISTICS')

    createReplaceView (session, 'SYSINFO.SPINLOCKSTATISTICSRESET',
        """AS SELECT *
           FROM SYSDD.SPINLOCKSTATISTICSRESET
        INTERNAL""")

    createSysinfoSynonym (session, 'SPINLOCKSTATISTICSRESET')

    createReplaceView (session, 'SYSINFO.TASKGROUPSTATISTICS',
        """AS SELECT *
           FROM SYSDD.TASKGROUPSTATISTICS
        INTERNAL""")

    createSysinfoSynonym (session, 'TASKGROUPSTATISTICS')

    createReplaceView (session, 'SYSINFO.TASKGROUPSTATISTICSRESET',
        """AS SELECT *
           FROM SYSDD.TASKGROUPSTATISTICSRESET
        INTERNAL""")

    createSysinfoSynonym (session, 'TASKGROUPSTATISTICSRESET')

    createReplaceView (session, 'SYSINFO.TASKLOADBALANCINGINFORMATION',
        """AS SELECT *
           FROM SYSDD.TASKLOADBALANCINGINFORMATION
        INTERNAL""")

    createSysinfoSynonym (session, 'TASKLOADBALANCINGINFORMATION')

    createReplaceView (session, 'SYSINFO.TASKLOADBALANCINGTASKGROUPSTATES',
        """AS SELECT *
           FROM SYSDD.TASKLOADBALANCINGTASKGROUPSTATES
        INTERNAL""")

    createSysinfoSynonym (session, 'TASKLOADBALANCINGTASKGROUPSTATES')


    createReplaceView (session, 'SYSINFO.TASKLOADBALANCINGTASKMOVES',
        """AS SELECT *
           FROM SYSDD.TASKLOADBALANCINGTASKMOVES
        INTERNAL""")

    createSysinfoSynonym (session, 'TASKLOADBALANCINGTASKMOVES')

    createReplaceView (session, 'SYSINFO.TRANSACTIONHISTORY',
        """AS SELECT *
           FROM SYSDD.TRANSACTIONHISTORY
        INTERNAL""")

    createSysinfoSynonym (session, 'TRANSACTIONHISTORY')

#   TRANSACTIONS_NEW because of SYSDBA.TRANSACTIONS
    createReplaceView (session, 'SYSINFO.TRANSACTIONS',
        """AS SELECT *
           FROM SYSDD.TRANSACTIONS_NEW
        INTERNAL""")

#   PTS 1132642
#   createSysinfoSynonym (session, 'TRANSACTIONS')
    stopIfSqlNotOK (session, 'grant select on sysinfo.transactions to public')


    createReplaceView (session, 'SYSINFO.UNLOADEDSTATEMENTS',
        """AS SELECT *
           FROM SYSDD.UNLOADEDSTATEMENTS
        INTERNAL""")

    createSysinfoSynonym (session, 'UNLOADEDSTATEMENTS')

    createReplaceView (session, 'SYSINFO.VERSION',
        """AS SELECT *
           FROM SYSDD.VERSION_NEW
        INTERNAL""")

    createSysinfoSynonym (session, 'VERSION')

    createReplaceView (session, 'SYSINFO.LOGSTATISTICS',
        """AS SELECT *
           FROM SYSDD.LOGSTATISTICS
        INTERNAL""")

    createSysinfoSynonym (session, 'LOGSTATISTICS')

    createReplaceView (session, 'SYSINFO.LOGSTATISTICSRESET',
        """AS SELECT *
           FROM SYSDD.LOGSTATISTICSRESET
        INTERNAL""")

    createSysinfoSynonym (session, 'LOGSTATISTICSRESET')

    createReplaceView (session, 'SYSINFO.LOGINFORMATION',
        """AS SELECT *
           FROM SYSDD.LOGINFORMATION
        INTERNAL""")

    createSysinfoSynonym (session, 'LOGINFORMATION')

    createReplaceView (session, 'SYSINFO.LOGQUEUESTATISTICS',
        """AS SELECT *
           FROM SYSDD.LOGQUEUESTATISTICS
        INTERNAL""")

    createSysinfoSynonym (session, 'LOGQUEUESTATISTICS')

    createReplaceView (session, 'SYSINFO.LOGQUEUESTATISTICSRESET',
        """AS SELECT *
           FROM SYSDD.LOGQUEUESTATISTICSRESET
        INTERNAL""")

    createSysinfoSynonym (session, 'LOGQUEUESTATISTICSRESET')

    createReplaceView (session, 'SYSINFO.SERVERTASKS',
        """AS SELECT *
           FROM SYSDD.SERVERTASKS
        INTERNAL""")

    createSysinfoSynonym (session, 'SERVERTASKS')

    switchToDefaultSchema (session, options)


    rc = sqlRC (session, 'COMMIT WORK')
    loaderCmd (session, 'INSTALLATION OFF')

    switchToDefaultUser (session, options)
